/**
 * @summary     initVisuGraph
 * @description Display the app in new tab after starter and substarter has been selected in start page.
 * @version     
 * @file        initVisuGraph.js
 * @author      Alexan Andrieux
 * @contact     <a href="mailto:alexan.andrieux@inria.fr"> Alexan Andrieux</a>
 * @category
 *
 * @copyright 
 *
 * This source file is free software, under AGLP License
 * 
 * This source file is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 * 
 * For details please refer to: <a href="http://www.gnu.org/licenses/agpl-3.0.html"></a>
 */

/**
 * Load the visualization page composed by:
 * - A menu for save session, load session and modify settings of the graph (layout, style)
 * - A cytoscape graph area
 * - Data tables for graph elements (nodes and edges)
 * - A vizmapper that allow to highlight wanted information and modify some behavior of the graph area
 * - A panel that allow to visualize and work with nodes sequences (concatenate, export, comment, format) 
 * 
 * @module initVisuGraph
 * 
 * @requires <a href='http://blog.jquery.com/2011/09/12/jquery-1-6-4-released/'>jQuery 1.6.4+</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-1.4.2.min.js"></script>
 * 
 * @requires <a href='http://jqueryui.com/download'>jQuery UI 1.8.16+</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-ui-1.8.2.custom.min.js"></script>
 * 
 * @requires <a href='http://cytoscape.github.io/cytoscape.js/'>cytoscape 2.0.4+</a>
 * @requires <a href='http://layout.jquery-dev.net/documentation.cfm'>jquery.layout</a>
 * @requires <a href='https://github.com/eligrey/FileSaver.js/'>FileSaver</a>
 * @requires <a href='https://github.com/eligrey/Blob.js/'>Blob</a>
 * @requires DataTables 
 */

$(function () {
"use strict";	
	/**
	 * Build and display some elements:
	 * - Header with name and logo
	 * - Menu 1 
	 * - Vizmapper in east panel in a tab
	 * - Vizstyle in east panel in a tab
	 * - DataTables for nodes and edges in west panel in two different tabs
	 * - Graph area
	 * 
	 * Build south panel and Menu 2, but keep hidden;
	 *  
	 * Finally, call drawGraph() function to build cytoscape graph object.
	 * @for drawGraph
	 * 
	 */
	
	/**
	* Path of initial theme and config file
	* @attribute VISUAL_STYLE_URI
	* @readOnly
	* @static
	* @final
 	* @type String
 	* @default "./style/initStyle.tjson"
 	* @private
 	*/
	var VISUAL_STYLE_URI = "./style/initStyle.tjson";
	
	/**
	* Path of initial theme and config file
	* @attribute CONFIG_URI
	* @readOnly
	* @static
	* @final
 	* @type String
 	* @default "./config.json"
 	* @private
 	*/
	var CONFIG_URI = "./config.json";
	
	/**
	* Graph data object come from init module
	* 
	* localStorage used : graphData
	* 
	* @attribute GRAPH_LOCALSTORAGE
 	* @type Object
 	* @private
 	*/
	var GRAPH_LOCALSTORAGE = JSON.parse(localStorage.getItem('graphData'));
	
	
	var visualStyles = {},
	    config = {}, 
	    requests = [],
	    $cy = $("#cy"),
	  	$body = $('body');
	  	
	/* Load default visual style (initStyle.tjson) */		
	requests.push(
		$.getJSON(VISUAL_STYLE_URI, function(tjson) {
			visualStyles = tjson;
		})
		.fail(function() {
			alert("Error: The theme file can't be loaded. Make sure the files"+" initStyle.tjson".bold()+" is present in"+ " /style".bold() +" directory or your browser accept local files.");
			$("#dataTable").empty();
		}), 
		$.getJSON(CONFIG_URI, function(file) {
			config = file;
		}).fail(function() {
			alert("Error: The config file can't be loaded. Make sure the files"+" config.json".bold()+" is present or your browser accept local files." );
			$("#dataTable").empty();
	}));
	
	/* Build and display app elements   */	
	$.when.apply($, requests).done(
		function () {				
			$(document)
				/* Add gif picture when graph is loading: loading class add ajax-loader.gif (style.css) */
				.ajaxStart(function(){
		        	$("#cy").addClass("loading");			
		    	})
		    	.ready(function(){
					$("#dataTable").tabs();
						
					$('#side').prepend("<ul> </ul>");
					 if(config.vizstyle.display){
					 	$('#side ul').append('<li>\
												<a id="vizstyle_link" href="#vizstyle">\
													<span>Visual Style</span>\
												</a>\
											</li>');
						$("#side_content").append(" <div id='vizstyle' class='content'> </div>");
					 };
						  	
					if(config.vizmapper.display){
						$('#side ul').append('<li>\
												<a id="vizmapper_link" href="#vizmapper">\
														<span>Vizmapper</span>\
												</a>\
											</li>');
						$("#side_content").append(" <div id='vizmapper' class='content'> </div>");
					};
							
					/* Append header div for subtabs */
					$("#side_content > .content").each(function(){
						var id = $(this).attr("id");
							$("#side_header").append('<div id="' + id + '_header" class="header"></div>');
					});
		
					$("#side").tabs();
			
					/* Initialize menu for graph elements and check default values */
					var $menu_northPaneVisuGraph = $("#menu_northPaneVisuGraph");
					$menu_northPaneVisuGraph.cytoscapeMenu("graph", config);		
					$("#nodesLabel").children(":first").addClass("ui-menu-checked");
					$("#edgesLabel").children(":first").addClass("ui-menu-checked");
					$("#default").children(":first").addClass("ui-menu-checked");
										
					/* Initialize all panel (jquery.layout-latest.js) */
					$("body").layout({
						slideTrigger_open:		"mouseenter",
						north__paneSelector:  "#paneNorth",
						north__resizable:    false,
						north__slidable:     false,
						north__closable:     false,
						north__minSize: 28,
						east__paneSelector:  "#side",
						east__size:  "20%",
						east__minSize: 220,
						west__paneSelector:  "#dataTable",
						west__size:  "22%",
						west__minSize: 290,
						south__paneSelector:  "#paneSouth",
						south__size:  "27%",
						south__minSize: 250,
						south__initClosed:   true,
						south__spacing_closed:   28,
						south__slidable:     false,
	
					});
					
					if($(window).width()<=990){
						$("#paneNorth > h2").hide();
						$("body").layout().sizePane("north", "28px");
						$("body").layout().close("east");
						$("body").layout().close("west");
						
						setTimeout(function() {
							$cy.cyNavigator("resize");
							cy.resize();
							cy.fit();
						}, 200);
						
								
					}
					
					
					$(".ui-layout-resizer-south").attr("id", "sequences-resizer");
						
					/* Initialize menu for sequences and check default values */
					var $menu_southPaneVisuGraph = $("#menu_southPaneVisuGraph");
					$menu_southPaneVisuGraph.cytoscapeMenu("sequence", config);
					$("#fasta").children(":first").addClass("ui-menu-checked");
					
			}); 
			
			/* Fix for galaxy for toggler mouseover and mouse click*/
			$(".ui-layout-toggler").on('mouseenter mouseover mouseleave mouseout', function(e) {
				e.data = "";
				e.stopPropagation();
			});
		
			drawGraph(GRAPH_LOCALSTORAGE, visualStyles, config);
			
	 });
	
	/**
	* Replace alert browser with jQuery UI dialogue.
	*
	* @event alertUI
	* @param {string} message the message alert.
	*/
	window.alert = function(message){
			    $(document.createElement('div'))
			        .attr({title: 'Warning', 'class': 'alert'})
			        .html(message)
			        .dialog({
			            buttons: {OK: function(){$(this).dialog('close');}},
			            close: function(){$(this).remove();},
			            draggable: true,
			            modal: true,
			            resizable: false,
			            width: 'auto'
			        });
	};
	
	/**
	* Action on resize window
	*
	* @event resizeWindow
	*/
	$(window).resize(function() {
			if($(window).width()>=990){
					$("#paneNorth > h2").show();
					$("body").layout().sizePane("north", "100px");		
			}else{
				$("#paneNorth > h2").hide();
				$("body").layout().sizePane("north", "28px");
				$("body").layout().close("east");
				$("body").layout().close("west");
			}
			setTimeout(function() {
				sideTaskBarSupport();
				$cy.cyNavigator("resize");
				cy.resize();
			}, 400);
			scrollBarSupport(400, config);

	});    

	/**
	* Initialize, add events and display data tables of edges and nodes.  
	* 
	* Localstorage used : graphData
	* 
	* @class initPaneWestContent
	* @constructor
	* @param {array} tableNodes list of data nodes for datatable.
	* @param {array} tableEdges list of data edges for datatable.
	*/
	function initPaneWestContent(tableNodes, tableEdges){
		var $tables ={},
			height = $('#cy').height(),
			tableWidth = $('#dataTable').width();  
            
    	/* nodes tab */
    	$('#graphNodes').append('<table cellpadding="0" cellspacing="0" border="0" class="display dataGraph" id="dataGraph_nodes"></table>');
    	
    	var dataColumnsNodes = [ { "sTitle": "ID" }, { "sTitle": "Length" }, { "sTitle": "Current Coverage" }];
    	 var nbCoverageFileNodes = tableNodes[0].length-dataColumnsNodes.length-2;
    	  
    	for(var i=1; i<nbCoverageFileNodes; i++){
			dataColumnsNodes.push({ "sTitle": "File_"+i+" Coverage" }); 		
    	}
    	  
    	dataColumnsNodes.push({"sTitle": "In Degree"}, {"sTitle": "Out Degree"}, { "sTitle": "Sequence" });
    	 
    	var nbColumns = dataColumnsNodes.length-1; 
    	
		$tables.nodes = $('#dataGraph_nodes').dataTable({
			"bJQueryUI" : true,
			"bProcessing" : true,
			"bPaginate" : false,
			"bDeferRender": true,
			//"sPaginationType": "full_numbers",
			"aaData" : tableNodes,
			"aoColumns" : dataColumnsNodes,
        	"sScrollX" : tableWidth,
        	"bScrollCollapse": true
        	
		});
		
		
		$("#dataGraph_nodes_filter").find("input").keyup(function() {
			var highlightStr = $(this).val();
			var highlightStrLen = highlightStr.length;
	
			if(highlightStrLen != 0){
				$("#dataGraph_nodes > tbody > tr").each(function() {
					var trSeq = $(this).children()[nbColumns];
					var sequence = $(trSeq).text().toUpperCase();
					$(trSeq).highlight(highlightStr);
				});
			}
			

		});
		
		
		
		/* Hide the sequence columns */
		//$tables.nodes.api().column( nbColumns ).visible( false );
		$('#dataGraph_nodes tr').each(function(){
			$(this).children(":last").hide();
		});
		$("#dataGraph_nodes_wrapper thead tr").each(function(){
			$(this).children(":last").hide();
		});
			
		
		
		$("#dataGraph_nodes_filter").before("<button>Select list</button>");
		
		
		/* Define action after click on the button select list */
		$("#dataGraph_nodes_filter").prev().click(function(){
			//cy.nodes().unselect();
			$("#cy").addClass("loading");
			setTimeout(function(){
				$('#dataGraph_nodes tr').each(function(){
				
					var nodeId = $(this).children(":first").text();
					//strNodes += 'node#'+nodeId+', ';
					cy.$("#"+nodeId).select();
					
					/*Add highlight for each motif founded*/
					var $biojs = $("#divSequence_"+nodeId);
					if($biojs.get(0)!= undefined){
						var biojsData = $biojs.data("biojsData");
						biojsData.removeAllHighlights();
						var trSeq = $(this).children()[nbColumns];
						var rep =$(trSeq).data("repeat");
						for(var i=0; i<rep; i++){
							var arrayStart = $(trSeq).data("start");
							var arrayStop = $(trSeq).data("stop");
						
							biojsData.addHighlight( { "start": arrayStart[i], "end": arrayStop[i], "color": "white", "background": "red", "id": "aaa" } );

						};
						
					}
					

				});
				//var selector = strNodes.substring(8,strNodes.length-2);
				//cy.elements(selector).select();
				
				
				$("#cy").removeClass("loading");
				
			},100);
			
			
			
		});
		
		

		initPaneWestContent.prototype.onClickNodes();
		$("#graphNodes_link").click(function(e) {
			//cy.edges(":selected").unselect();
			var elShowing = $("#dataGraph_nodes_info").text();
			$("#nb_element").text(elShowing);
			scrollBarSupport(50, config);
			//e.stopImmediatePropagation();
			
		});
		
		/*edges tab*/
	    $('#graphEdges').append('<table cellpadding="0" cellspacing="0" border="0" class="display dataGraph" id="dataGraph_edges"></table>');
	    
	    var dataColumnsEdges = [ { "sTitle": "ID" },{ "sTitle": "Source" },{ "sTitle": "Target" },{ "sTitle": "Direction" },{ "sTitle": "Current Coverage" }];
	    var nbCoverageFileEdges = tableEdges[0].length-dataColumnsEdges.length;
	   	for(var j=1; j<nbCoverageFileEdges; j++){
	    	dataColumnsEdges.push({ "sTitle": "File_"+j+" Coverage" });  		
	    }
	    	    
		$("#graphEdges_link").click();
		$tables.edges = $('#dataGraph_edges').dataTable({
			"bJQueryUI" : true,
			"bProcessing" : true,
			"bPaginate" : false,
			"bDeferRender": true,
			//"sPaginationType": "full_numbers",
			"aaData" : tableEdges,
			"aoColumns" : dataColumnsEdges,
			"sScrollX" : tableWidth,
			"bScrollCollapse": true

		}); 
		
		//Adjust columns size
		$("#graphEdges_link").css({ width: $("#graphEdges_link").parent().width() });
    	$tables.edges.fnAdjustColumnSizing();  
  
		initPaneWestContent.prototype.onClickEdges();
		$("#graphEdges_link").click(function(e) {
			//cy.nodes(":selected").unselect();
			var elShowing = $("#dataGraph_edges_info").text();
			$("#nb_element").text(elShowing);
			scrollBarSupport(50, config);
			//e.stopImmediatePropagation();
		});


  		initPaneWestContent.prototype.onResize();
  		

  		/* Add updateTable to cy core */
    	 cytoscape( "collection", "updateTable", function(idTable, index, columns, value, max){
					 var oTableElements =$(idTable).dataTable();
					 oTableElements.fnUpdate( value, index, columns ); // Single cell
					 if(this._private.group == "nodes"){
					 	 cy._private.maxNodesCoverage = max;
					 }else if(this._private.group = "edges"){
					 	 cy._private.maxEdgesCoverage = max;
					 }
		});	
    	
		$("#graphNodes_link").click();
		//Adjust columns size
		$("#graphNodes_link").css({ width: $("#graphNodes_link").parent().width() });
    	$tables.nodes.fnAdjustColumnSizing();     
	}
	
	initPaneWestContent.prototype ={
		
		/**
		* If length is present in data node, function find and return the biggest length of sequence in the list of node
		* else function calculate the length sequence of each node and return the biggest length calculated.
		* Function fill tableNodes with data of each node: 
		* - id
		* - length
		* - sequence
		* - current avg coverage
		* - coverage file
		*
		* @method initNodesData
		* @param {object} nodes list of nodes.
		* @param {array} tableNodes array with all data nodes.
		* @return {array} array with max length of sequence and  max value of avg coverage.
		* @private
		*/
		initNodesData: function(nodes, tableNodes) {
			var nodeCount = nodes.length,
			elData  = [],
			lengthMax = 0,
			maxNodesCoverage =0;
	           	
	        for (var i = 0; i < nodeCount; i++) {
	            	
	        	var nodeData =  nodes[i].data,
	        	    nodeId = nodeData.id,
					seqNodesCoverage = nodeData.coverageValSelected,
					seqNodesLength = nodeData.length,
					nodesInFrom = cy.nodes("#"+nodeId).indegree(),
					nodesOutTo = cy.nodes("#"+nodeId).outdegree();
				  
				
				/* If length is not present in data node, calculate the length sequence of each node */
				if( seqNodesLength==undefined){
					seqNodesLength = nodeData.sequence.length;
					cy.$("#"+nodeId).data("length", seqNodesLength);
				}
	
				/* Find the biggest sequence length */
				if( seqNodesLength > lengthMax){
						lengthMax = seqNodesLength;						
				};
					
				if(seqNodesCoverage!="none" || seqNodesCoverage!=undefined){
					if( seqNodesCoverage >  maxNodesCoverage){
							maxNodesCoverage = seqNodesCoverage;						
					};
				}
					
					
				/* tableNodes is filled with data node */
				tableNodes[i] =[
					nodeId,
					seqNodesLength,
					nodeData.coverageFileSelected	
				];
				  	
				  		
				/*Add columns for each coverage file */
				if(nodes[0].data.coverage != undefined){
				  	var nbFileCoverage =nodes[0].data.coverage.length;
					if(nodeData.coverage.length!=0){
					  var j =0;
				      while(j<nbFileCoverage){
				      	tableNodes[i].push(nodeData.coverage[j].avg_coverage);
				        j++;
				      }
					  }else{
					  	var j =0;
					  	while(j<nbFileCoverage){
					  		tableNodes[i].push("none");
					  	j++;
					  }
					  		
					}
				}
				  	
				
						
					
				tableNodes[i].push(nodesInFrom, nodesOutTo);
				/*Add columns sequences */
				tableNodes[i].push(nodeData.sequence);
					   	
			};
	           
	    	return [lengthMax,  maxNodesCoverage];
		}, 
		
		/**
		* Function fill tableEdges with data of each edge:
		* - id 
		* - source
		* - target
		* - direction
		* - current avg coverage
		* - coverage file
		*
		* @method initEdgesData  
		* @param {object} nodes list of edges.
		* @param {array} tableEdges array with all data edges.
		* @return {int} maxEdgesCoverage max value of avg coverage. 
		* @private
		*/
		initEdgesData : function(edges, tableEdges){
			var edgeCount = edges.length,
				maxEdgesCoverage = 0;
	 			
	       	for (var i = 0; i < edgeCount; i++) {
	            	
	        	var  edgeData = edges[i].data,
	             	 edgeCoverage = edgeData.coverageValSelected;     
					
				if(edgeCoverage!="none" || edgeCoverage!=undefined){
					if( edgeCoverage > maxEdgesCoverage){
						maxEdgesCoverage = edgeCoverage;						
					};
				}
				  
				 /*tableEdges is filled with data node */
				tableEdges[i] =[
				 	edgeData.id,
					edgeData.source,
					edgeData.target,
					edgeData.direction,
					edgeData.coverageFileSelected		
				 ];
				  	
				 /* Add columns for each coverage file */
				if(edges[0].data.coverage != undefined){
					var nbFileCoverage =edges[0].data.coverage.length;
					if(edgeData.coverage.length!=0){
						var j =0;
				         while(j<nbFileCoverage){
				           tableEdges[i].push(edgeData.coverage[j].avg_coverage);
				           j++;
				        }
					 }else{
					  	var j =0;
					  	while(j<nbFileCoverage){
					  		tableEdges[i].push("none");
					  		j++;
					  	}
					  		
					}
				}
				  
			}
			return maxEdgesCoverage;
	
		},
		
		/**
		* Click on edge in dataTable highlight the line and select edge on graph.
		* Multi selection is allowed with the hot key Shif+Left Click. 
		* 
		* @event onClickEdges
		*/
		onClickEdges : function (){	
		    	$('#dataGraph_edges tr').click( function(e){   	   				    	   				
		    		var $this = $(this);
					if ( !$this.hasClass('selected')) {
						if(!e.shiftKey){
							cy.edges(":selected").unselect();
						     //$this.parent().find(".selected").removeClass('selected');
						}
						$this.addClass('selected');
		    	   		cy.edges('#'+$this.children(":first").text()).select();
		          	}else{
		          			$this.removeClass('selected');
		    	   			cy.edges('#'+$this.children(":first").text()).unselect();
		          		}
		        });
		},
	
		/**
		* Click on node in dataTable highlight the line and select node on graph. 
		* Multi selection is allowed with the hot key Shif+Left Click. 
		* 
		* @event onClickNodes
		*/
		onClickNodes : function (){
			$('#dataGraph_nodes tr')
				.mousedown(function(){ //Add loading image for big nodes
					var nodeId = $(this).find(".sorting_1").text();
					if(cy.$("#"+nodeId).data("length")/25 > 10){
							$("#cy").addClass("loading");
					}
				})
				.click( function(e){   //select node(s) on graph	   				    	   				
		    		var $this = $(this);
						if ( !$this.hasClass('selected')) {
							if(!e.shiftKey){
						       cy.nodes(":selected").unselect();
						           	//$this.parent().find(".selected").removeClass('selected');
						    }
						    $this.addClass('selected');
		    	   			cy.nodes('#'+$this.children(":first").text()).select();
		          		}else{
		          			$this.removeClass('selected');
		    	   			cy.nodes('#'+$this.children(":first").text()).unselect();
		    	   			$("#cy").removeClass("loading");
		          		}
	          });   	
		},
		
		/**
		* Resize cytoscape navigator and ajust size for scrollBar support
		* @event onResize
		*/
		onResize:function(){
			/* add support for resize window */
			$(".ui-layout-resizer-west")
				/* namespace allow to define mouseup event after mousemove/drag */
				.on('mousedown.namespace', function() {
							$(document).one('mouseup', function() {
								    scrollBarSupport(400, config);
								    setTimeout(function(){
								    	$cy.cyNavigator("resize");
								  	 	cy.resize();
								  	}, 400);	
									$(document).off('mousedown.namespace');
							});
												  
				});
		}
	};
        
	/**
	* Initialize, add events and display vistyle and vizmapper.
	* 
	* @class initPaneEastContent
	* @param {object} config object build with the configuration file config.json. This file define the behavior of the app.
	* @constructor
	*/
	//TODO:Change click event define in click event
	function initPaneEastContent (config){
		var saveNodes,
			saveEdges,
			saveHoldRoot=false,
			firstClickVizmapper = undefined,
			firstClickVizstyle = true,
			vizstyleIsActive = false,
			vizmapperIsActive = false,
			firstClickNodes = false,
			firstClickEdges = false,
			tabNodesIsActive = false,
			tabEdgesIsActive = false,
			vizstyleDisplay = config.vizstyle.display,
			vizmapperDisplay = config.vizmapper.display,
			save;
				
		/* vizstyle is display only with advanced vizmapper*/
		if(vizstyleDisplay){
			
			/* add sub tabs*/
			$("#vizstyle_header").append('<div id="vizstyle_tabs">\
												<ul>\
													<li><a id=VizstyleCore_link>Core</a></li>\
													<li><a id=VizstyleNodes_link>Nodes</a></li>\
													<li><a id=VizstyleEdges_link>Edges</a></li>\
												</ul>\
											  </div>');
						
			$("#vizstyle_tabs").tabs();	
			
			/* Define action on click vizstyle tab */ 
			$("#vizstyle_link").click(function(e){
						
				$("#vizmapper_tabs").hide();
				$("#vizstyle_tabs").show();	
			
				$("#VizstyleCore_link").parent().addClass("ui-state-active ui-tabs-selected").siblings().removeClass("ui-state-active ui-tabs-selected");
				
				if(vizmapperIsActive && tabNodesIsActive){
					saveNodes = $("#vizmapper.content").vizMapSave();
					saveHoldRoot=$("#hold_root").prop("checked");
					firstClickVizmapper = false;	
				};
				
				if(vizmapperIsActive && tabEdgesIsActive){
					saveEdges = $("#vizmapper.content").vizMapSave();
					firstClickVizmapper = false;	
				};  
														
				firstClickVizstyle = false;
					
				setTimeout(function(){	
					sideTaskBarSupport();
				},250);
				
				vizmapperIsActive = false;
				vizstyleIsActive = true;
				
				if(cy._private.centered){
					$("#focusNode_Vizstyle").prop("checked", true);
				}else{
					$("#focusNode_Vizstyle").prop("checked", false);
				}
				
				e.stopImmediatePropagation();
			});
			
			/* Define core clock event*/
			var vizstyle = $("#vizstyle.content").vizStyle("core");
			$("#VizstyleCore_link").click( function(e){
				vizstyle.initCore();
				$("#VizstyleCore_link").parent().addClass("ui-state-active ui-tabs-selected").siblings().removeClass("ui-state-active ui-tabs-selected");
				tabNodesIsActive = false;
				tabEdgesIsActive = false;
				e.stopImmediatePropagation();
			});
				
			/* Define nodes clock event*/
			$("#VizstyleNodes_link").click( function(e){
				vizstyle.initNodes();
				$("#VizstyleNodes_link").parent().addClass("ui-state-active ui-tabs-selected").siblings().removeClass("ui-state-active ui-tabs-selected");
				tabNodesIsActive = true;
				tabEdgesIsActive = false; 
				e.stopImmediatePropagation();	              
			});
					
			/* Define edgesclock event*/ 
			$("#VizstyleEdges_link").click( function(e){
				vizstyle.initEdges();
				$("#VizstyleEdges_link").parent().addClass("ui-state-active ui-tabs-selected").siblings().removeClass("ui-state-active ui-tabs-selected");	
				tabNodesIsActive = false;
				tabEdgesIsActive = true;
				e.stopImmediatePropagation();                           
			});
			
		
		}
		
		//TODO: updateInitialOpts update really opts object 
		if(vizmapperDisplay){
			/* Add sub tabs*/
			$("#vizmapper_header").append('<div id="vizmapper_tabs">\
												<ul>\
													<li><a id=VizmapperNodes_link>Nodes</a></li>\
												 </ul>\
											</div>');
					
			if(cy._private.maxEdgesCoverage!=0){
				$("#vizmapper_tabs ul").append("<li><a id=VizmapperEdges_link>Edges</a></li>");
			}
						
			$("#vizmapper_tabs").tabs();
			
		
			/* Define action on click on vizmapper tab */
			$("#vizmapper_link").click(function(e){
					if(vizstyleDisplay==false || vizstyleIsActive){
					
						$("#vizstyle_tabs").hide();	
						$("#vizmapper_tabs").show();
						
						$("#VizmapperNodes_link").parent().addClass("ui-state-active ui-tabs-selected").siblings().removeClass("ui-state-active ui-tabs-selected");

						if(firstClickVizmapper == undefined){
							initPaneEastContent.prototype.initVizmapperNodes(config);
							firstClickVizmapper =true;
							tabNodesIsActive = true;
							tabEdgesIsActive = false;
							firstClickNodes = true;
							firstClickEdges = undefined;
							
							 	
						}else {
							$("#VizmapperNodes_link").parent().addClass("ui-state-active ui-tabs-selected").siblings().removeClass("ui-state-active ui-tabs-selected");
				
							$("#vizmapper.content").vizMapRestore(saveNodes, config);
		
							tabNodesIsActive = true;
							tabEdgesIsActive = false;
							firstClickNodes = false;
	
						};
				
					};

				vizmapperIsActive = true;
				vizstyleIsActive = false;
				scrollBarSupport(0, config);
				
				e.stopImmediatePropagation();
				});
				
				
				/* Define action on click on nodes tab*/
				$("#VizmapperNodes_link").click(function(e) {
				
						$(this).parent().addClass("ui-state-active ui-tabs-selected").siblings().removeClass("ui-state-active ui-tabs-selected");
						if (tabEdgesIsActive && vizmapperIsActive) {
				
							if (firstClickEdges != undefined) {//save if already click edges
								saveEdges = $("#vizmapper.content").vizMapSave();
							};
				
							if (firstClickNodes != undefined) {//restore if already click node

								$("#vizmapper.content").vizMapRestore(saveNodes, config);
								firstClickNodes = false;
							} else {
								initPaneEastContent.prototype.initVizmapperNodes(config);
								firstClickNodes = true;
							};
				
							tabEdgesIsActive = false;
							tabNodesIsActive = true;
						};

						scrollBarSupport(0, config);
						e.stopImmediatePropagation();
					}); 
	
									
				/* Define action on click on edges tab*/
				$("#VizmapperEdges_link").click(function(e) {
				
						$(this).parent().addClass("ui-state-active ui-tabs-selected").siblings().removeClass("ui-state-active ui-tabs-selected");
				
						if (tabNodesIsActive && vizmapperIsActive) {
							if (firstClickNodes != undefined) { //save if alraedy click nodes
								saveNodes = $("#vizmapper.content").vizMapSave();
								saveHoldRoot = $("#hold_root").prop("checked");
							};
				
							if (firstClickEdges != undefined) { //restore if alredy click edges
								$("#vizmapper.content").vizMapRestore(saveEdges, config);
								firstClickEdges = false;
							} else {
				
								initPaneEastContent.prototype.initVizmapperEdges(config);
								firstClickEdges = true;
							};
				
							tabEdgesIsActive = true;
							tabNodesIsActive = false;
						};
				
						scrollBarSupport(0, config);
						e.stopImmediatePropagation();
					});
		}
			
	  	/* Support for resize window and scrollbar appearance */ 
	 	initPaneEastContent.prototype.onResize();
	 	sideTaskBarSupport();	
		
	};

	initPaneEastContent.prototype ={
		/**
		* Constructor for nodes vizmapper in east pane.
		* 
		* @method initVizmapperNodes
		* @param {object} config object build with the configuration file config.json. It define the behavior of the app. 
		*/
		initVizmapperNodes : function (config) {
			/* Get nodes style */
			var nodes = cy.nodes(), 
				initCssNodes = nodes.css(), 
				initType = "gradient";
	
			//cy._private.vizmapperType = initType;
	
			if(config.vizmapper.pre_configured == undefined){
				config.vizmapper.pre_configured = {};
				config.vizmapper.pre_configured.default_nodes_map =["#FFF 0% 1 ellipse 30", "#000 100% 1 ellipse 30"];
				 
			}
			/* Generate and display vizmapper for nodes*/
			if (cy._private.maxNodesCoverage != 0) {
				/*Order is important in property and attrCss: the first element of property is associated  with the first element of attrCss*/
				//TODO: in vizmapper associat picker with attrCss and not with property
				$("#vizmapper.content").vizMap({
					group : "nodes",
					property : ["Length", "Coverage"], //length or/and coverage
					controlPoints : config.vizmapper.pre_configured.default_nodes_map,
					attrCss : ["shapes", "colors"],
					type : "gradient",
					tableOfNodes : '#dataGraph_nodes',
					tableOfEdges : '#dataGraph_edges',
					sequenceHeader : '.ui-sequences-visible',
					//change:function(){}
				}, config);
	
			} else {
				$("#vizmapper.content").vizMap({
					group : "nodes",
					property : ["Length"],
					controlPoints : [initCssNodes.backgroundColor + " 0% " + initCssNodes.opacity + " " + initCssNodes.shape + " 20", initCssNodes.backgroundColor + " 50% " + initCssNodes.opacity + " " + initCssNodes.shape + " 30", initCssNodes.backgroundColor + " 100% " + initCssNodes.opacity + " " + initCssNodes.shape + "40 "],
					attrCss : ["shapes"],
					type : initType,
					tableOfNodes : '#dataGraph_nodes',
					tableOfEdges : '#dataGraph_edges',
					sequenceHeader : '.ui-sequences-visible',
					//change:function(){}
				}, config);
			}
	
		},
			
		/**
		* Constructor for nodes vizmapper in east pane
		* 
		* 
		* @method initVizmapperEdges
		* @param {object} config object build with the configuration file config.json. It define the behavior of the app 
		*/
		initVizmapperEdges : function (config) {
			
			/*Get edges style */
			var edges = cy.edges(),
			/*initCssEdges = edges.css(),*/
			initType = "gradient";
	
			//cy._private.vizmapperType = initType;
			
			if (cy._private.maxEdgesCoverage != 0) {
				/* generate and display vizmapper for edges */
				$("#vizmapper.content").vizMap({
					group : "edges",
					property : ["Coverage"], //coverage only
					controlPoints : config.vizmapper.pre_configured.default_edges_map,
					attrCss : ["colors"],
					type : "gradient",
					tableOfNodes : '#dataGraph_nodes',
					tableOfEdges : '#dataGraph_edges',
					sequenceHeader : '.ui-sequences-visible',
					//change:function(){}
				}, config);
			}
	
		},
		
		/**
		* Resize cytoscape navigator and ajust size for scrollBar support
		* @event onResize
		*/
		onResize:function(){
			$(".ui-layout-resizer-east")
				/* namespace allow to define mouseup event after mousemove/drag */
				.on('mousedown.namespace', function() {
							$(document).one('mouseup', function() {
								    scrollBarSupport(400, config);
								   setTimeout(function(){
								    	$cy.cyNavigator("resize");
								    	cy.resize();
								    }, 400);
									$(document).off('mousedown.namespace');
							});
												  
				})
				.mouseover(function(){
						if($("#vizmapper.content").children()[0] != undefined && $("#vizmapper.content").width()==0){
									scrollBarSupport(400, config);
						}
			});	
		}
		
    };    


    /**
	* Build the graph with graph data object come from init.
	* Initialize dataTables in west panel.
	* Initialize vizmapper in east panel.
	* Define events on graph area.  
	* 
	* LocalStorage used : sessionName, sessionData, fromVisu
	* 
	* @class drawGraph
	* @constructor
	* @param {object} cyNetwork graph description with some data
	* @param {object} vsDict style css description 
	* @param {object} config object build with the configuration file config.json. It define the behavior of the app 
	*/
    function drawGraph(cyNetwork, vsDict, config) {
     	var defaultStyle = vsDict.style,
            rootId = undefined,
            cyRight = undefined,
            cyLeft= undefined,
            dataNodes ={},
            dataEdges ={};
            	
		/* Define root node ID */
        if(cyNetwork.starter != undefined){
        	rootId = cyNetwork.starter.data.id;
        }
            
        /*Define position of extensions in visu*/
        if(cyNetwork.position != undefined){
           cyRight = cyNetwork.position.right;
           cyLeft = cyNetwork.position.left;
        }
            
        $cy
        .cytoscape({
                				
			layout: { 
				name: config.default_layout, //initial layout
				roots: rootId, // id of root of the tree
				rightChildren: cyRight, // id of the first right node
				leftChildren: cyLeft, //id of the first left node
				//ready:function(){}, 
				start: function(){
					//$("#cy").addClass("loading");
				}

			},
				
            elements: {
            	nodes: cyNetwork.nodes,
                edges: cyNetwork.edges
            },
            
            style: (function () {    
            	var curStyleObj = cytoscape.stylesheet(),
                    elementCount = defaultStyle.length;

                /*Apply Css style define in tjson*/        
				for (var i = 0; i < elementCount; i++) {

					/*Exception style for root node*/
					if (defaultStyle[i].selector == "root:unselected") {
						defaultStyle[i].selector = "#" + rootId + ":unselected";
					} else if (defaultStyle[i].selector == "root:selected") {
						defaultStyle[i].selector = "#" + rootId + ":selected";
					} else if (defaultStyle[i].selector == "root") {
						defaultStyle[i].selector = "#" + rootId;
					}
					
					/* Style for other nodes */
					curStyleObj.selector(defaultStyle[i].selector).css(defaultStyle[i].css);

				}
                return curStyleObj; 
          })(),
          
          ready: function () {
          	window.cy = this;
            
            /*Define root node in cy_private*/
            if(rootId != undefined){
            	cy._private.root = cy.nodes("#"+rootId);
   				cy._private.root.data().previousCss = cy._private.root.css();
            }
   			
   			/* check good layout in menu */
   			$("#menu_northPaneVisuGraph").find("#"+this._private.layout.options.name).children(":first").addClass("ui-menu-checked");

			var nodes = this.nodes(),
			    edges = this.edges(),
			    root = this.nodes("#"+rootId),
				widthNodes = 0,
			 	heightNodes = 0,
			 	seq_PaneSouth = null,
			 	tableNodes =[],
			 	tableEdges =[],
			 	dataNodes = GRAPH_LOCALSTORAGE.nodes,
			 	dataEdges = GRAPH_LOCALSTORAGE.edges;
    		
			/* Apply label on elements*/
			nodes.css("content", "data(id)");
			edges.css("content", "data(direction)");

			
			
			/* Apply init functions and save maximums in cy_private (used by vizmapper)*/
			var initAndMaxNodes = initPaneWestContent.prototype.initNodesData(dataNodes, tableNodes);
 			this._private.maxNodesLength = initAndMaxNodes[0];
 			this._private.maxNodesCoverage = initAndMaxNodes[1];
			var initAndMaxEdges = initPaneWestContent.prototype.initEdgesData(dataEdges, tableEdges);	
			this._private.maxEdgesCoverage = initAndMaxEdges;		
			initPaneWestContent(tableNodes, tableEdges);			
			initPaneEastContent(config);
				  
  	  		
  	  		/*Define elements event on mouseover/mouseout*/
  	  		drawGraph.prototype.onMouseoverEl();
			drawGraph.prototype.onMouseoutEl();
  	  		
  	  		/*Save the initial css style of elements*/
			nodes.data().previousCss = nodes.css();
			edges.data().previousCss = edges.css();
			
			/*First, hide opposite direction edges*/
			var edges_ff = [],
				edges_rr = [];		
			
			edges.each(function () {
				if (this.data("direction")=="RR" || this.data("direction")=="rr" ){
					edges_rr.push(this);
					this.hide();
				}else if((this.data("direction")=="rf" || this.data("direction")=="RF") && this.data("source")>this.data("target")){
					edges_rr.push(this);
					this.hide();
				}else if ((this.data("direction")=="fr" || this.data("direction")=="FR") && this.data("source")<this.data("target")){
					edges_rr.push(this);
					this.hide();
				};
			});
			
			/*Define edges and nodes events*/	
			drawGraph.prototype.onSelectEdges();
			drawGraph.prototype.onUnselectEdges();
			
			cy._private.orderingSelection =[];
			drawGraph.prototype.onSelectNodes();
			drawGraph.prototype.onUnselectNodes();
			
			drawGraph.prototype.onMousedownNodes();
			drawGraph.prototype.onGrabNodes();
			
			/*Define graph area events*/
			drawGraph.prototype.onResize();
			drawGraph.prototype.onResizeSouthPanel();
			drawGraph.prototype.onMouseWheel(); 
			
			
			$cy
				/*Add cytoscape panzoom buttons (see cytoscape.js documentation) */
				.cytoscapePanzoom();
				
			if(config.with_navigator){ 		
				/*Add cytoscape navigator area (see cytoscape.js documentation) */
				$cy.append('<div class="cytoscape-navigator"> </div>')
				.cyNavigator({
					container: $cy.children(".cytoscape-navigator"),
					 viewLiveFramerate: false,
					 thumbnailEventFramerate: 1,
					  thumbnailLiveFramerate: false,
				});
			};
			
			/*Add cytoscape contextual menu (see cytoscape.js documentation) */
			if(config.with_cxtmenu){ 	
				$cy.cytoscapeCxtmenu({
		      						selector: 'node',
		      						commands: [{
										content: 'Connect',
										select: function(){
											$("#cy").cytoscapeEdgehandles('start', this.id());
										}
									},
									{
		          						content: 'Delete',
		          						select: function(){
		            						this.remove();
		          						}
		       			 			}
		       			 		]
		    		});
    		}
    		/*Add cytoscape Edgehandles function (see cytoscape.js documentation)*/
    		if(config.with_edgehandles){ 	
    			drawGraph.prototype.onEdgeHandles();
    		}
    		
    		/*Display tabs in function of config file*/
			if(config.vizmapper.display){ 	
				$("#vizmapper_link").click();
				$("#VizmapperEdges_link").click();
				$("#VizmapperNodes_link").click();
				
				$("#reset[class=vizmapper]").click(function() {
					scrollBarSupport(0, config);
				});
			}
			
			if(config.vizstyle.display){
				$("#vizstyle_link").click();
			}
			
			if(!config.vizstyle.display && config.vizmapper.display){
				$("#vizmapper_link").off("click");
			}else if(config.vizstyle.display && !config.vizmapper.display){
				$("#vizstyle_link").off("click");
			} 
			
			/*Apply session parameters if it's a session file*/
			if(localStorage.getItem("sessionName") !=undefined && localStorage.getItem('sessionData') != undefined){
						var sessionData = JSON.parse(localStorage.getItem('sessionData'));
						var currentTab = $("#side").find(".ui-tabs-selected").children();
						$("#vizmapper_link").click();
						$("#VizmapperEdges_link").click();

						var edges = cy.edges(), 
							cssEdges = edges.css();
						
						
						$("#vizmapper.content").vizMapRestore(sessionData.edges, config);

						$("#VizmapperNodes_link").click();

						var nodes = cy.nodes(), cssNodes = nodes.css();

						$("#vizmapper.content").vizMapRestore(sessionData.nodes, config);

						/* Restore the current tab after save*/
						if (currentTab.get(0) == $("#vizstyle_link").get(0) ) {
							$("#vizstyle_link").click();
							$("#core_link").click();
						} else if (currentTab.get(0)  == $("#vizmapper_link").get(0)  && currentTab.get(1)  == $("#VizmapperEdges_link").get(0) ) {
							$("#vizmapper_link").click();
							$("#VizmapperEdges_link").click();
						}

						var nbPositions = sessionData.position.length;
						var i = 0;
						
						nodes.each(function(){
							/* restore position */
							if(i<nbPositions){
								this.position(sessionData.position[i]);
								i++;
							}
							
							/*restore format, highlights and annotations for sequences of each node*/
							var arrayBiojsLen = sessionData.biojs.length;
							var nodeId = this.data("id");
							for(var j = 0; j<arrayBiojsLen; j++){
									var self = sessionData.biojs[j];
									var savedNodeId = self.id;
									if(nodeId == savedNodeId){
										/* Create new biojs sequence*/
										var sequenceWidth = parseInt($("#paneSouth").width()/8.2);
										var $onSequenceSelectionChanged = $("#onSequenceSelectionChanged");
										
										$("#sequences_content").append('<div id="divSequence_'+nodeId+'" class="ui-sequences ui-sequences-visible"><div>');	
										$("#divSequence_"+nodeId).data("nodeId",nodeId); //Associate $("#divSequence") to a node id		
										var biojs = new Biojs.Sequence({
												sequence : this.data("sequence"),
												coverage : this.data("coverage"),
												target : "divSequence_"+nodeId,
												format : self.format,
												id : nodeId,
												fontSize: '18px',
												columns: {size:sequenceWidth},
												//annotations: self.annotations,
												highlights: self.highlights
						    				});	
		  									
		  								/*re-add all annotations*/
		  								var nbAnnotation = self.annotations.length;
		  								for(var n =0; n<nbAnnotation; n++){
		  									//add empty anntotation if the index is not good
		  									if(n<self.annotations[n].index){
												biojs.addAnnotation({
													name : undefined,
													index : n,
													html : "",
													color : undefined,
													regions : undefined
												});
											}
											
											//add annotations saved
			  								biojs.addAnnotation({
												name : self.annotations[n].name,
												index : self.annotations[n].index,
												html : "",
												color : self.annotations[n].color,
												regions : self.annotations[n].regions
											});
											
											
		  								}
		  								
		  								/* Define events onSelectionChange for biojs sequences created*/
		  								biojs.onSelectionChange(function( e ) {// Display position of selection
												$(".ui-sequences-selected").show();
							      				$onSequenceSelectionChanged.text("In node "+nodeId+" interval selected: " + e.start + ", " + e.end );
							      				$onSequenceSelectionChanged.data( "id", nodeId);
							      				$onSequenceSelectionChanged.data( "selectedSequence", [e.start, e.end]);					
							  			});
							  			
							  			/* Define events onAnnotationClicked for biojs sequences created and annotation restored*/
							  			
							  			biojs.onAnnotationClicked(function(e) {
											var target = e.target;
											var id = target.attr("index");
											var classId = nodeId +"_" + id;
											var $className = $("."+target.attr("name"));
											
											
											if ($("#annotationpicker_"+classId)[0] == undefined) {
												$className.parent().append('<div class="color-chooser annotation-chooser" id="annotationpicker_' + classId + '"style="position:absolute;display:none;"></div>');
											}
											
											$('#annotationpicker_' + classId).picker({
												onShow : function(colpkr) {
			
												},
												onChangeColor : function(col) {
													var colorSelected = "#"+$.picker.hsbToHex(col);
													biojs._annotations[id].color = colorSelected;
													$className.attr("color", colorSelected).css("border-color", colorSelected);
													$('.annotation-chooser').css("backgroundColor", colorSelected);
												},
												onChangeName : function() {
													biojs._annotations[id].name = $(this).data("picker").name;
												},
												onChangeText : function() {
													biojs._annotations[id].html = $(this).data("picker").text;
												},
												onHide : function(colpkr) {
													$('#annotationpicker_' + classId).hide();
												},
												onSubmit : function (col) {
													var colorSelected = "#"+$.picker.hsbToHex(col);
													biojs._annotations[id].color = colorSelected;
													$className.attr("color", colorSelected).css("border-color", colorSelected);
													$('.annotation-chooser').css("backgroundColor", colorSelected);
													biojs._annotations[id].name = $(".sequencepicker").data("picker").name;
													biojs._annotations[id].html = $(".sequencepicker").data("picker").text;
													$('#annotationpicker_' + classId).hide();
												}
												
											});
											
											var color = $.Color($className.attr("color")).toHexString();
											var top = $className.position().top - 10;
											var left = $className.position().left;
				
											$('#annotationpicker_' + classId).css({
												top : top,
												left : left,
												backgroundColor : $className.attr("color"),
												display : "block"
											}).pickerSetValues(color, 1, undefined, biojs._annotations[id].name, biojs._annotations[id].html).pickerShow();
							  			
							  			});	
							  				
							  			/*reset selection and hide restore sequences*/
							  			biojs.setSelection(0, 0); //remove selection when unselect node
										$("#divSequence_"+nodeId).removeClass("ui-sequences-visible").hide(); 
										
							  			/* save biojs sequences in nodes and the div divSequence_* */	
							  			$("#divSequence_"+nodeId).data("biojsData",biojs);
		  								this.data("biojsData",biojs);	
							        
							        	j++;           
						    		}
					    		};
						});
						
						if(rootId != undefined){
			            	cy.nodes("#"+rootId).position(sessionData.position[nbPositions-1]);
			            	cy._private.root = cy.nodes("#"+rootId);
			            }
						$('#currentSession').val(localStorage['sessionName']);
						
						if($('#edgesLabel').children(":first").hasClass("ui-menu-checked")){
							edges.css("content", "data(direction)");
						}else{
							edges.css("content", "");
						}
						
						if($('#nodesLabel').children(":first").hasClass("ui-menu-checked")){
							nodes.css("content", "data(id)");
						}else{
							nodes.css("content", "");
						}
			}

			
	 		localStorage["fromVisu"] = true;

	 		if(config.vizmapper.display){ 	
				$("#vizmapper_link").click();
				$("#VizmapperEdges_link").click();
				$("#VizmapperNodes_link").click();
			}
			
			if(config.vizstyle.display){
				$("#vizstyle_link").click();
			}
			
			$("#cy").removeClass("loading");
	 		
        }

      });
    
	
    }
    
    drawGraph.prototype= {
    	/**
    	 * Define action when edges are selected:
    	 * - Apply css style
    	 * - Select in DataTable
    	 * - Show edges if is hidden
    	 * - build south panel
    	 * - Display south panel
    	 * - Hide useless menu elements 
    	 * 
    	 * @event onSelectEdges
    	 */
    	onSelectEdges : function(){
    		/*Define select event on edges*/
			cy.edges().select(function(){
				this.data().hidden = this.hidden();
				this.data("previousCss", this.css());	
				this.css({	//Apply a css style selection
					"line-color": "blue",
					"source-arrow-color": "blue",
					"target-arrow-color": "blue",
					"opacity": 1,
				});
				
				if($('#edgesLabel').children(":first").hasClass("ui-menu-checked")){
					this.css("content", "data(direction)");
				}else{
					this.css("content", "");
				}
				
				var self =this;
				var td = $('#dataGraph_edges tr').find(".sorting_1");
				td.each(function(){ //Selection in datatable
					var $this = $(this);
					if($this.text()== self.data("id")){
						$this.parent().addClass("selected");
					}
				});
				
				if(this.hidden()){ //Display the edges hidden
					this.show();
				}
				
				var edgeCoverage = this.data("coverage"),
					edgeId = this.data("id"),
					edgeTarget = this.data("target"),
					edgeSource = this.data('source');
				
					
				var str = "<span class=pane-south-data-attribute>ID:</span><span class=pane-south-data-value selected>"+edgeId+
						  " </span><span class=pane-south-data-attribute> Source:</span><span class=pane-south-data-value selected>" +edgeSource+
						  "</span><span class=pane-south-data-attribute> Target:</span><span class=pane-south-data-value selected>"+edgeTarget+
						  "</span><span class=pane-south-data-attribute> Coverage:</span><span class=pane-south-data-value selected>";
				

				if(edgeCoverage != undefined){
					if(edgeCoverage != 'none'){
						for(var j=0; j<edgeCoverage.length; j++){
							str += '</span><span class=pane-south-data-value file='+edgeCoverage[j].id+'>'+edgeCoverage[j].id + ':'+edgeCoverage[j].avg_coverage+' </span>'; 
						}
					}else{
						str += '</span><span class=pane-south-data-value file=none>none </span>'; 
					}
					
				}
				
				$("#sequences_content ").append(" <di class='edgesData'><pre>"+str+"</pre></div>" );
				
				/* Set style of coverage data displayed in panel south in function of coverage file selected */
	    		$(".pane-south-data-value[file="+this.data("coverageFileSelected")+"]").removeClass("unselected").addClass("selected");
	    		$(".pane-south-data-value[file]").not("[file="+this.data("coverageFileSelected")+"]").removeClass("selected").addClass("unselected");
				$("#onSequenceSelectionChanged").text(' ');//Empty header for edges
				
				if(cy.nodes(":selected").length==0){
					$("#menu_southPaneVisuGraph ul").children().not('#holdSouth').hide(); //Hide node menu for edges
				}
				
				setTimeout(function(){
						cy.elements(":unselected").css("opacity", 0.5);	//Reduce opacity of others elements
				},100);

				var $layout = $body.layout();
				
				if($layout.south.state.isClosed){//Resize panel for task bar only if south panel is not lock
					setTimeout(function(){
						southTaskBarSupport();
						cy.resize(); 
						//cy.fit();
					}, 260);
				}
				
				$layout.open("south"); //Show south panel
				
						
				scrollBarSupport(400, config); //Support scrollbar disappear for east and west panel when south pane is opened
			});
    	},
    	
    	/**
    	 * Define action when edges are unselected:
    	 * - Restore css style
    	 * - Unselect in DataTable
    	 * - Hide edges if is hidden before
    	 * - Hide south panel
    	 * 
    	 * @event onUnselectEdges
    	 */
    	onUnselectEdges: function(){
    		//// restore edges visual style ////
			cy.edges().unselect(function(){
				$(".edgesData").remove();
				
				this.css(this.data("previousCss")); //Restore previous css style
				
				$('#dataGraph_edges tr').filter(".selected").removeClass('selected');//Remove selection elements in dataTable
				
				/*Restore node menu*/
				//$("#menu_southPaneVisuGraph ul").children().not('#holdSouth').show();
				
				if(this.data().hidden){ //Hide edges if they are hide before
					this.hide();
				}	
				
				//setTimeout(function(){
					//if(cy.elements(":selected").length == 0){
						cy.elements(":unselected").css("opacity", 1); // Restore opacity
						$body.layout().close("south"); //Close south panel
						
						/*if(!$("#holdSouth").hasClass('ui-state-active')){ //Resize panel for task bar only if south panel is not lock
							setTimeout( function() {
									southTaskBarSupport();
									//cy.fit(100);
							}, 230);
						}*/
						
						scrollBarSupport(400, config);
						cy.resize(); 
					//}
				//},50);
			});
    	},
    	
    	/**
    	 * Define action when nodes are selected:
    	 * - Apply css style
    	 * - Save order of selection
    	 * - Select in DataTable
    	 * - build south panel
    	 * - Display south panel
    	 * - Show all menu elements 
    	 * 
    	 * @event onSelectNodes
    	 */
    	onSelectNodes: function(){
	    	cy.nodes().select(function(){
	    			$("#cy").addClass("loading");
					/*needed for concatenate function*/
					cy._private.orderingSelection.push(this);	
							
					this.data("previousCss", this.css());	
					this.css({//Apply css style
						"border-color": "blue",
						"opacity" : 1,
						"borderWidth": 2,
						"text-outline-color": "blue"
										
					});
					
					$("#menu_southPaneVisuGraph ul").children().show(); //show menu elements hidden for edges
					
					if($('#nodesLabel').children(":first").hasClass("ui-menu-checked")){
						this.css("content", "data(id)");
					}else{
						this.css("content", "");
					}
							
					var self =this;
					var td = $('#dataGraph_nodes tr').find(".sorting_1");
					td.each(function(){ //select in datatable
						var $this = $(this);
						if($this.text()== self.data("id")){
								$this.parent().addClass("selected");
						}
					});
					
	
					var nodeSequence = this.data("sequence"),
						nodeCoverage = this.data("coverage"),
						nodeId = this.data("id"),
						$onSequenceSelectionChanged = $("#onSequenceSelectionChanged"),
						selectSeq = undefined;
	
					var sequenceWidth = parseInt($("#paneSouth").width()/8.2);
					var formatSelected = $("#setFormat > ul").find(".ui-menu-checked").siblings("label").text(); 
		
					if (this.data("biojsData") == undefined){
						$("#sequences_content").append('<div id="divSequence_'+nodeId+'" class="ui-sequences ui-sequences-visible"><div>');	
						$("#divSequence_"+nodeId).data("nodeId",nodeId); //Associate $("#divSequence") to a node id		
							 /*First time the node is selected : Initialize the sequence of selected node (see biojs.js documentation) */
							var biojs = new Biojs.Sequence({
								sequence : nodeSequence,
								coverage : nodeCoverage,
								target : "divSequence_"+nodeId,
								format : formatSelected,
								id : nodeId,
								fontSize: '18px',
								columns: {size:sequenceWidth},
		    				});	

							biojs.onSelectionChange(function( e ) {// Display position of selection
								$(".ui-sequences-selected").show();
			      				$onSequenceSelectionChanged.text("In node "+nodeId+" interval selected: " + e.start + ", " + e.end );
			      				$onSequenceSelectionChanged.data( "id", nodeId);
			      				$onSequenceSelectionChanged.data( "selectedSequence", [e.start, e.end]);					
			  				});		
			                   
		    		}else{
		    				/*The node has been already selected before: Set sequence(s) with sequence(s) of selected node(s)*/
		    				$("#divSequence_"+nodeId).addClass("ui-sequences-visible").show();	
		    		}		
		    					
		  			//TODO: Maybe do the same with setSequence()
		  			$onSequenceSelectionChanged.text("In node "+nodeId+" interval selected: " +0 + ", " + 0 ); //Reset position selection 
	
		    		$("#divSequence_"+nodeId).data("biojsData",biojs);
		  			this.data("biojsData",biojs);
		
		  			/*Set style of coverage data displayed in function of coverage file selected */
		    		$(".pane-south-data-value[file="+this.data("coverageFileSelected")+"]").removeClass("unselected").addClass("selected");
		    		$(".pane-south-data-value[file]").not("[file="+this.data("coverageFileSelected")+"]").removeClass("selected").addClass("unselected");
		    		
		    		if(this.data("k") == undefined){ //Display concatenate button only if this action is possible
						$("#menu_southPaneVisuGraph ul").children("#ConcaSeq").hide();
					}
					
					if (cy._private.centered){//Option: focus on node selected    
						cy.center(this);
					};
		  					
					setTimeout(function(){
							cy.elements(":unselected").css("opacity", 0.5);	//Reduce opacity of others elements
					},100);
		  					
					var $layout = $body.layout();
					
					var timeOut = parseInt(260+this.data("length")/15); // Set time out in function of the count of characters in the node sequence
					if($layout.south.state.isClosed){//Resize panel for task bar only if south panel is not lock
						setTimeout(function(){
							southTaskBarSupport(); 
							//cy.fit();
						}, timeOut);
					}
					
					$layout.open("south");
							
					scrollBarSupport(400, config); // Support scrollbar appearance for east and west pane when south pane is opened
	    	   		cy.resize(); 
	    	   		
	    	   		setTimeout(function(){
						$("#cy").removeClass("loading");
					}, timeOut);

				});
		},
		
		/**
    	 * Define action when nodes are unselected:
    	 * - Restore css style
    	 * - Unselect in DataTable
    	 * - Hide south panel
    	 * 
    	 * @event onUnselectNodes
    	 */
		onUnselectNodes:function(){
			cy.nodes().unselect(function () {
				var self = this;
				
				cy._private.orderingSelection = [] ; // Empty list of ordering selection of nodes
								
								
				self.css(self.data("previousCss")); //Restore previous css style
								
				$('#dataGraph_nodes tr').filter(".selected").removeClass('selected'); // Remove selection in dataTable
		
				var biojs = self.data("biojsData");
				var nodeId= self.data("id");
			
				biojs.setSelection(0, 0); //remove selection when unselect node
				$("#divSequence_"+nodeId).removeClass("ui-sequences-visible").hide(); 	
				$(".ui-concatenated-sequences").remove();
				
				/*Close south panel only if all elements has been unselected*/
				//setTimeout(function(){
					//if(cy.elements(":selected").length == 0){		
						cy.elements(":unselected").css("opacity", 1); //Restore opacity
						$body.layout().close("south"); //close south panel
									
						/*if(!$("#holdSouth").hasClass('ui-state-active')){
							setTimeout( function() {
										southTaskBarSupport();
										//cy.fit(100);
							}, 300);
						}*/
					//}
				//},50);
				
				scrollBarSupport(400, config); //Support scrollbar 	disappear for east and west pane when south pane is opened 		
				cy.resize(); 
			});
		},
		
		/**
		* Apply style on graph elements when mouseover
		* 
		* @event onMouseoverEl
		*/
		//TODO:on/off in tjson files or theme files
		onMouseoverEl: function(){
			cy.on("mouseover touchover ", "edge:unselected", function() {
				this.data().previousCss = this.css();
	
				this.css({
					"text-outline-color": "blue",
					"line-color" : "#aaa",
					"target-arrow-color" : "blue",
					"source-arrow-color" : "blue"
				});
	
			});
			cy.on("mouseover touchover", "node:unselected", function() {
				this.data().previousCss = this.css();
					this.css({
						"text-outline-color": "blue",
						"border-color" : "blue",
						"background-color" : "grey",
						"border-width" : "2"
					});
				
			});
		},
		
		
		/**
		* Restore style on graph elements when mouseout 
		* 
		* @event onMouseoutEl
		*/
		//TODO:on/off in tjson files or theme files
		onMouseoutEl: function(){
			cy.on("mouseout touchout mouseup", "edge:unselected", function() {
					var previousCss = this.data("previousCss");
					this.css({
						"text-outline-color": previousCss.textOutlineColor,
						"line-color" : previousCss.lineColor,
						"target-arrow-color" : previousCss.targetArrowColor,
						"source-arrow-color" : previousCss.sourceArrowColor
					});
			});
			cy.on("mouseout touchout mouseup", "node:unselected", function() {
				var previousCss = this.data("previousCss");
				this.css({
					"text-outline-color": previousCss.textOutlineColor,
					"border-color" : previousCss.borderColor,
					"background-color" : previousCss.backgroundColor,
					"border-width" : previousCss.borderWidth
				});
			});
		},
		
		/**
		* Display loading image for big sequences (>250) 
		* 
		* @event onMousedownNodes
		*/
		onMousedownNodes: function(){
			//cy.on('mousedown', 'node', function(evt){
				/* Display loading if nodes length >250 because take time to generate all characters in panel south with biojs*/
				//if(this.data("length")/25 > 10){
					//$("#cy").addClass("loading");
				//}
			//});
		},
		
		/**
		* Off events onMouseoverEl() and onMouseoutEl() on grab and on drag nodes
		* 
		* @event onGrabNodes
		*/
		onGrabNodes: function(){
			cy.on("grab", function(){
				cy.off("mouseout mouseover touchout touchover");
					$(document).one('mouseup', function() {
					    drawGraph.prototype.onMouseoverEl();
					    drawGraph.prototype.onMouseoutEl();
					});
			});
		},
		
		/**
		* Update zoom level in taskbar
		* 
		* @event onMouseWheel
		*/
		onMouseWheel: function(){
			/*Show Zoom level in taskbar */ 
			$("#cy").on("mousewheel DOMMouseScroll",function(){
				var lvl = parseFloat(cy.zoom()).toFixed(2);
				$("#zoom_lvl").text('Zoom level: ' + lvl);
			});
		},
		
		/**
		* Off events onMouseoverEl() and onMouseoutEl() on resize graph area
		* 
		* @event onResize
		*/		
		onResize: function(){
			/*Off mouseover elements when resize a panel*/
		   $(".ui-layout-resizer").on('mousedown.namespace', function() {
						cy.off("mouseout mouseover touchout touchover");
					    $(document).one('mouseup', function() {
					    			drawGraph.prototype.onMouseoverEl();
					    			drawGraph.prototype.onMouseoutEl();
									cy.forceRender();
									$cy.cyNavigator("resize");
									cy.resize(); 
									//cy.panBy({x:0,});
						        	$(document).off('mousedown.namespace');  		
					    });
			 });
		},
		
		/**
		* Add scrollbar and taskbar support on resize south panel 
		* 
		* @event onResizeSouthPanel
		*/
		onResizeSouthPanel: function(){
			/*Support scrollbar for east and west panel and resize south panel to support taskbar*/
			$(".ui-layout-resizer-south").on('mousedown.namespace', function() {
					    $(document).one('mouseup', function() {	
					    			scrollBarSupport(0, config);
					    			//cy.resize(); 
					    			southTaskBarSupport();
						        	$(document).off('mousedown.namespace');  		
					    });
			 });
		},
		
		onEdgeHandles: function(){
			/**
			 * See cytoscape.js documentation :
			 * This function is not activated
			 *
			 * @event onEdgehandles
			 */
			 $cy.cytoscapeEdgehandles({
			 		lineType: "bezier",
			 		preview: true,
			 		handleSize: 12,
			 		handleColor: "orange",
			 		edgeType: function(){
			 			return "flat";
					},
					nodeParams: function(){
			 			return {
			 				classes: "intermediate"
			 			};
			 		},
					start: function( sourceNode ){
			 			//console.log("start(%o)", sourceNode);
			 		},
			 		complete: function( sourceNode, tathisrgetNodes, added ){
					 	//console.log("complete(%o, %o, %o)", sourceNode, targetNodes, added);
			 		},
			 		stop: function( sourceNode ){
			 			//console.log("stop(%o)", sourceNode);
				 	}
				 });
		
		}
		
		
		
    };
    
	
	/**
	* Adjust size of west and east panel with the taskbar 
	* @class sideTaskBarSupport
	* @constructor
	*/
    var sideTaskBarSupport= function (){
        var height = $('#cy').height();
        $('#side').css("height", height);
		var newHeight = height-$('.ui-layout-header').height()-$('#side > .ui-tabs-nav').height()-$('#paneTask').height()+10;
		   $('#side_content').css("height", newHeight);	
     };
     
     /**
	* Adjust size of south panel with the taskbar 
	* @class southTaskBarSupport
	* @constructor
	*/   
     var southTaskBarSupport = function (){
        var correctionHeight = $('#sequences_content').height()-$('#paneTask').outerHeight();
		$('#sequences_content').css("height", correctionHeight);
	};
	
		
	/**
	* Resize all panels when scrollbar has appeared/disappeared.
	* 
	* @class scrollBarSupport
	* @constructor
	* @param {int} timeout for the setTimeout function.
	* @param {object} config object build with the configuration file config.json. It define the behavior of the app. 
	*/
	var scrollBarSupport = function(timeout, config) {
		
		/* Support for east panel */
		if ($("#vizmapper.content").children().get(0) != undefined) {
			setTimeout(function() {
				var save = $("#vizmapper.content").vizMapSave();
				$("#vizmapper.content").vizMapUpdate(save, config);
			}, timeout);
		};

		/* Support for west panel */
		setTimeout(function() {
			var height = $('#cy').height();
			var tableHeight = height - $('#dataTable > .ui-tabs-nav').outerHeight()-3;
			$('#dataTable').css("height", height);
			$('#dataTable_content').css("height", tableHeight);

			// Set size of datatable for scrollbar support 
			if ($("#graphNodes_link").parent().hasClass("ui-state-active")) {
				var fgNodes = $("#graphNodes").find(".fg-toolbar")[0];
				var correctionHeightNodes = $(fgNodes).outerHeight();
				fgNodes = $("#graphNodes").find(".fg-toolbar")[1];
				$(fgNodes).hide();
				correctionHeightNodes += ($("#graphNodes").find(".dataTables_scrollHead").outerHeight());
				$("#graphNodes").find('.dataTables_scrollBody').css("height", $('#dataTable_content').height() - correctionHeightNodes);
			} else if ($("#graphEdges_link").parent().hasClass("ui-state-active")) {
				var fgEdges = $("#graphEdges").find(".fg-toolbar")[0];
				var correctionHeightEdges = $(fgEdges).outerHeight();
				fgEdges = $("#graphEdges").find(".fg-toolbar")[1];
				$(fgEdges).hide();
				correctionHeightEdges += ($("#graphEdges").find(".dataTables_scrollHead").outerHeight());
				$("#graphEdges").find('.dataTables_scrollBody').css("height", $('#dataTable_content').height() - correctionHeightEdges);
			};
			
			sideTaskBarSupport();

		}, timeout);
		
	};

});
