/**
 * @summary     cytoscapeMenu
 * @description Create Menu for cytoscape.js panel and Biojs.Sequence.js area. 
 * @version     
 * @file        cytoscape.menu.js
 * @author      Alexan Andrieux
 * @contact     <a href="mailto:alexan.andrieux@inria.fr"> Alexan Andrieux</a>
 * @category
 *
 * @copyright 
 *
 * This source file is free software, under AGPL License.
 * 
 * This source file is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 *
 * For details please refer to: <a href="http://www.gnu.org/licenses/agpl-3.0.html"></a>
 */

/** 
 * 
 * Build a menu for cytoscape panel. Two menu are available : 
 * - Menu1 : functions for the graph
 * - Menu2 : functions for nodes sequences 
 * 
 * Define a toggle methods for show/hide elements (nodes and edges) labels. 
 * 
 * @module cytoscapeMenu
 * @requires <a href='http://blog.jquery.com/2011/09/12/jquery-1-6-4-released/'>jQuery 1.6.4</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-1.4.2.min.js"></script>
 * 
 * @requires <a href='http://jqueryui.com/download'>jQuery UI 1.8.16</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-ui-1.8.2.custom.min.js"></script>
 * 
 * @requires toggler
 * @requires <a href='http://cytoscape.github.io/cytoscape.js/'>cytoscape 2.0.4+</a>
 * @requires <a href='https://code.google.com/p/jquery-menubar/'>jquery.menuBar</a> 
 * @requires Biojs.Sequence
 * 
 * @example
 * 		$(selector).cytoscapeMenu(typeOfMenu, configFile)
 */

(function($){
"use strict";
	/*import json graph*/
	var cyNetwork = JSON.parse(localStorage.getItem('graphData')); //or cy.json() with new cytoscape.js 
	
	/** 
	 * Define menuNorth, menuSouth and subMenuThemes into a CytoscapeMenu object 
	 * and associate methods with each menu elements.
	 * 
	 * 	@class CytoscapeMenu
	 *  @constructor
	 *  @param {string type type of menu (menu 1 or menu 2). 
	 *  @param {object} config the parsed config file. 
	 *
	 * 
	 */
	function CytoscapeMenu($el, config){
		this.$el = $el;
		var self = this;
		this.currentConfig = config;
		
		

		/**
		 * Define Menu 1:
		 * - Load > Load session file:  call loadSession() to restore old session saved in sjson file								  
		 * - Load > Start page : loading a new start page to load a new graph file (sjson or json)											 
		 * - Save > Save session file : call  saveSession() to save a session file (sjson) 								
		 * - Style > Show nodes labels : call labelNodesToggle to show/hide nodes labels (default: show)
		 * - Style > Show edges labels : call labelEdgesToggle to show/hide edges labels (default: show)													
		 * - Layout > Recalculate layout: re-draw the layout with initial position
		 * - Layout > Mechanism : set type of layout (default : hierarchical) 
		 * @type {object}  
		 * @property menuNorth
		 * @private
		*/
		var menuNorth = {
			items : [{
				/*Load file menu*/ 
				name : "Load file",
				open : function() {
	
				},
				items : [{
					name : "Load session file (vjson, sjson)",
					attr : {
						id : "upload_session"
					},
					selecton : function() {
						/* Append input file, hack for set the input file css style and define action on file change.*/
						if($('#upload_session').parent().find('input')[0]==undefined){
							$('#upload_session').parents("ul").parent().parent().prepend('<input type="file" id="hiddenSessionLoad" class="ui-menu-title ui-state-default" style="display:none" />');
							$('#hiddenSessionLoad').change( function(){				
										var files = $(this)[0].files,
									 		fileName = files[0].name;	 		
											localStorage["sessionName"]= fileName;
							      			self.loadSession(files); 				 	 
							});
						};
						
						setTimeout(function(){
							$('#hiddenSessionLoad').click();
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						},10);
					}
				}, {
					name : "Start page",
					attr : {
						id : "start_page"
					},
					selecton : function() {
						localStorage['fromVisu'] = true;
						window.location.href = 'index.html';
					}
				}]
			}, {
				/*Save file Menu*/
				name : "Save file",
				open : function() {
	
				},
				items : [
				{
				 name: "Save vizmapper (vjson)",
				 attr: { id:"download_vjson" },
				 selecton: function(){
						self.saveVizmapper();
						
						if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
				 	}
				 },
				{
					name : "Save session file (sjson)",
					attr : {
						id : "download_sjson"
					},
					selecton : function(e) {
						cy.nodes(":selected").unselect();
						self.saveSession();
						
						if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					}
				}]
			}, {
				/*Style menu*/
				name : "Style",
				open : function() {
	
				},
				items : [{
					name : "Show nodes labels",
					checkable : true,
					attr : {
						id : "nodesLabel"
					},
					selecton : function() {
						labelsToggle.prototype.labelNodesToggle(true);
						
						if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					},
					selectoff : function() {
						labelsToggle.prototype.labelNodesToggle(false);
						
						if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					}
				}, {
					name : "Show edges labels",
					checkable : true,
					attr : {
						id : "edgesLabel"
					},
					selecton : function() {
						labelsToggle.prototype.labelEdgesToggle(true);
						
						if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					},
					selectoff : function() {
						labelsToggle.prototype.labelEdgesToggle(false);
						
						if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					}
				}]
			}, {
				/*Layout menu*/
				name : "Layout",
				open : function() {
	
				},
				items : [{
					name : "Recalculate layout",
					attr : {
						id : " "
					},
					selecton : function() {
						cy.layout();
						
						if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					}
				}, {
					name : "Mechanism",
					attr : {},
					open : function() {
	
					},
					items : [{
						name : "Null",
						checkable : true,
						attr : {
							id : "null"
						},
						selecton : function() {
							$("#cy").addClass("loading");
							setTimeout(function(){	
								cy.layout({
										name : "null",
										stop:function(){
											$("#cy").removeClass("loading");
										}
									});
							}, 10);
							$("#null").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "Preset",
						checkable : true,
						attr : {
							id : "preset"
						},
						selecton : function() {
							$("#cy").addClass("loading");
							setTimeout(function(){	
								cy.layout({
										name : "preset",
										stop:function(){
											$("#cy").removeClass("loading");
										}
									});
							}, 10);
							$("#preset").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						 };
						}
					}, {
						name : "Random",
						checkable : true,
						attr : {
							id : "random"
						},
						selecton : function() {
							$("#cy").addClass("loading");
							setTimeout(function(){	
								cy.layout({
										name : "random",
										stop:function(){
											$("#cy").removeClass("loading");
										}
									});
							}, 10);
							$("#random").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "Grid",
						checkable : true,
						attr : {
							id : "grid"
						},
						selecton : function() {
							$("#cy").addClass("loading");
							setTimeout(function(){
								cy.layout({
									name : "grid",
									start: function(){
										$("#cy").addClass("loading");
									},
									stop:function(){
										$("#cy").removeClass("loading");
									}
								});
							}, 10);
							
							$("#grid").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "Arbor",
						checkable : true,
						attr : {
							id : "arbor"
						},
						selecton : function() {
							$("#cy").addClass("loading");
							setTimeout(function(){	
								cy.layout({
										name : "arbor",
										stop:function(){
											$("#cy").removeClass("loading");
										}
									});
							}, 10);
							$("#arbor").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "Circle",
						checkable : true,
						attr : {
							id : "circle",
							
						},
						selecton : function() {
							$("#cy").addClass("loading");
							setTimeout(function(){	
								cy.layout({
										name : "circle",
										stop:function(){
											$("#cy").removeClass("loading");
										}
									});
							}, 10);
							$("#circle").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "Cose",
						attr : {
							id : "cose"
						},
						checkable : true,
						selecton : function() {
							$("#cy").addClass("loading");
							setTimeout(function(){	
								cy.layout({
										name : "cose",
										stop:function(){
											$("#cy").removeClass("loading");
										}
									});
							}, 10);
							$("#cose").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "Breadthfirst",
						attr : {
							id : "breadthfirst"
						},
						checkable : true,
						selecton : function() {
							var rootId = undefined;
	
							if (cyNetwork.starter != undefined) {
								rootId = cyNetwork.starter.data.id;
							}

							$("#cy").addClass("loading");
							setTimeout(function(){	
								cy.layout({
										name : "breadthfirst",
										roots: cy.$("#"+rootId),
										stop:function(){
											$("#cy").removeClass("loading");
										}
									});
							}, 10);
							$("#breadthfirst").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "Hirarchical",
						attr : {
							id : "hierarchical"
						},
						checkable : true,
						selecton : function() {
							var rootId = undefined, cyRight = undefined, cyLeft = undefined;
	
							if (cyNetwork.starter != undefined) {
								rootId = cyNetwork.starter.data.id;
							}
	
							if (cyNetwork.position != undefined) {
								cyRight = cyNetwork.position.right;
								cyLeft = cyNetwork.position.left;
							}
							$("#cy").addClass("loading");
							setTimeout(function(){	
								cy.layout({
									name : "hierarchical",
									roots : rootId, // id of root of the trees
									rightChildren : cyRight, // id of the first right node
									leftChildren : cyLeft, //id of the first left node
									stop:function(){
										$("#cy").removeClass("loading");
									}
								});
							}, 10);
							
							$("#hierarchical").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "Concentric",
						attr : {
							id : "Concentric"
						},
						checkable : true,
						selecton : function() {
							$("#cy").addClass("loading");
							setTimeout(function(){	
								cy.layout({
										name : "concentric",
										stop:function(){
											$("#cy").removeClass("loading");
										}
									});
							}, 10);
							$("#Concentric").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-north").hasClass("active")){
								$(".menu-button-north").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
							}
					}]
				}
				//TODO:implement this
				/*,{
				 name: "Settings",
				 attr: { },
				 selecton: function(){
				 open_settings(["Null","Preset","Random","Grid","Arbor","Circle","Radial*","Breadthfirst", "Hierarchical"]);
				 }
	
				 }*/
				]
			}, {
				name : "Help",
				selecton : function() {
					window.open('doc/doc.html');
				}
			}]
		};
		
		this.menuNorth = menuNorth; 
	
		
		/**
		 * Define new sub-menu for menu 1 to support themes ( themes suport must be activated in config file )
		 * - Style > Visualization: set the theme file (tjson) used to the graph
		 * @type {object}  
		 * @property subMenuThemes
		 * @private
		 */
		var subMenuThemes = {
			name : "Visualization",
			attr : {},
			open : function() {
	
			},
			items : [{
				name : "Default",
				checkable : true,
				attr : {
					id : "default"
				},
				selecton : function() {
					var visualStyles = {}, requests = [];
					requests.push($.getJSON("./style/initStyle.tjson", function(file) {
						visualStyles = file.style;
					}));
					$.when.apply($, requests).done(function() {
						applyStyle(visualStyles);
						$("#cy").removeClass("loading");
						$("#default").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
					});
	
				}
			}, {
				name : "Cytoscape",
				checkable : true,
				attr : {
					id : "cytoscape"
				},
				selecton : function() {
					var visualStyles = {}, requests = [];
					requests.push($.getJSON("./style/cytoscape.tjson", function(file) {
						visualStyles = file.style;
					}));
					$.when.apply($, requests).done(function() {
						applyStyle(visualStyles);
						$("#cy").removeClass("loading");
						$("#cytoscape").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
					});
	
				}
			}, {
				name : "Default2",
				checkable : true,
				attr : {
					id : "cytoscape"
				},
				selecton : function() {
					var visualStyles = {}, requests = [];
					requests.push($.getJSON("./style/default.tjson", function(file) {
						visualStyles = file.style;
					}));
					$.when.apply($, requests).done(function() {
						applyStyle(visualStyles);
						$("#cy").removeClass("loading");
						$("#cytoscape").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
					});
	
				}
			}]
		};
		this.subMenuThemes =subMenuThemes; 
	
		
		/**
		 * Define Menu 2:
		 * - Concatenate:  call concatenateSeq.init() to concatenate at least two sequences								  
		 * - Export : export a file with sequences displayed											 
		 * - Add > Annotation : call addAnnotation() of Biojs.Sequence to add annotation on selected sequences 								
		 * - Add > Highlight : call addHighlight() of Biojs.Sequence to add highlight on selected sequences 	
		 * - Remove > All : call removeAllAnnotations() and unHighlightAll() of Biojs.Sequence to remove all annotation and all highlight of sequence displayed								
		 * - Remove > All annotation: call removeAllAnnotations() of Biojs.Sequence to remove all annotation of sequence displayed
		 * - Remove > All highlight : call unHighlightAll() of Biojs.Sequence to remove all highlight of sequence displayed																 
		 * @property menuSouth
		 * @type {object} 
		 * @private
		*/
		
		var menuSouth = {
			items : [{
				/*Concatenate button*/
				name : "Concatenation",
				attr : {
					id : "ConcaSeq"
				},
				selecton : function() {
					var collection = cy._private.orderingSelection;
					concatenateSeq.init(collection);
					
					if($(".menu-button-south").hasClass("active")){
						$(".menu-button-south").removeClass("active");
					    $(".flexnav-show").removeClass("flexnav-show");
					};
				}
			}, {
				/*Export button*/
				name : "Export",
				attr : {
					id : "exportSeq"
				},
				selecton : function() {
					

					var sequenceExport = $(".ui-sequences-visible").find("pre");
					var textSequenceExport = $(sequenceExport).text();
				
					var blob = new Blob([textSequenceExport], {
						type : "text/plain;charset=utf-8"
					});
					if ($('#fasta').children(":first").hasClass("ui-menu-checked")) {
						saveAs(blob, "squencesExported.fasta");
					} else if ($('#codata').children(":first").hasClass("ui-menu-checked")) {
						saveAs(blob, "squencesExported.codata");
					} else if ($('#pride').children(":first").hasClass("ui-menu-checked")) {
						saveAs(blob, "squencesExported.pride");
					} else if ($('#raw').children(":first").hasClass("ui-menu-checked")) {
						saveAs(blob, "squencesExported.txt");
					}
	
					if($(".menu-button-south").hasClass("active")){
						$(".menu-button-south").removeClass("active");
						$(".flexnav-show").removeClass("flexnav-show");
					};
				}
				
			},{
				/*Set format sequence menu*/
				name : "Set",
				attr : {
					id : "SetSeq"
				},
				open : function() {
	
				},
				items : [{
					name : "Format",
					attr : {
						id : "setFormat"
					},
					open : function() {
	
					},
					items : [{
						name : "FASTA",
						checkable : true,
						attr : {
							id : "fasta"
						},
						selecton : function() {
							cy.nodes(":selected").each(function() {
								this.data("biojsData").setFormat("FASTA");
								$(".pane-south-data-value[file="+this.data("coverageFileSelected")+"]").removeClass("unselected").addClass("selected");
		    					$(".pane-south-data-value[file]").not("[file="+this.data("coverageFileSelected")+"]").removeClass("selected").addClass("unselected");
							});
							
							if($(".ui-concatenated-sequences").get(0)!=undefined){
								$(".ui-concatenated-sequences").data("biojsData").setFormat("FASTA");
							}
							
							$("#fasta").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "CODATA",
						checkable : true,
						attr : {
							id : "codata"
						},
						selecton : function() {
							cy.nodes(":selected").each(function() {
								this.data("biojsData").setFormat("CODATA");
								$(".pane-south-data-value[file="+this.data("coverageFileSelected")+"]").removeClass("unselected").addClass("selected");
		    					$(".pane-south-data-value[file]").not("[file="+this.data("coverageFileSelected")+"]").removeClass("selected").addClass("unselected");
							});

							if($(".ui-concatenated-sequences").get(0)!=undefined){
								$(".ui-concatenated-sequences").data("biojsData").setFormat("CODATA");
							}
							
							$("#codata").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
														
							if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "PRIDE",
						checkable : true,
						attr : {
							id : "pride"
						},
						selecton : function() {
							cy.nodes(":selected").each(function() {
								this.data("biojsData").setFormat("PRIDE");
							});
							if($(".ui-concatenated-sequences").get(0)!=undefined){
								$(".ui-concatenated-sequences").data("biojsData").setFormat("PRIDE");
							}
							
							$("#pride").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}, {
						name : "RAW",
						checkable : true,
						attr : {
							id : "raw"
						},
						selecton : function() {
							cy.nodes(":selected").each(function() {
								this.data("biojsData").setFormat("RAW");
							});
							
							if($(".ui-concatenated-sequences").get(0)!=undefined){
								$(".ui-concatenated-sequences").data("biojsData").setFormat("RAW");
							}
							
							$("#raw").siblings().find(".ui-menu-checked").removeClass("ui-menu-checked");
							
							
							if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
							};
						}
					}]
				}]
			},{
				/*Add annotation and highlight menu*/
				name : "Add",
				attr : {
					id : "addSeqElements"
				},
				open : function() {
	
				},
				items : [{
					name : "Annotation",
					attr : {
						id : "addAnnotation"
					},
					selecton : function() {
						var nodeId = $("#onSequenceSelectionChanged").data("id");
						var $biojs = $("#divSequence_"+nodeId);

						$biojs.each(function() {
							var biojsData = $(this).data("biojsData");					
							var nbAnnotation = biojsData.opt.annotations.length;
							//index need to be define here to keep the right value
							//var biojsIndex = parseInt($(".annotation").attr("index"))+1;
							//console.log(parseInt($(this).find(".annotation").attr("index")))
							var name = $(this).find(".annotation").attr("name");
							for(var index = 0; index<nbAnnotation; index++){
								if(!biojsData.checkAnnotationName(name)){
									break;
								};
							}
													
							biojsData.addAnnotation({
								name : "annotation_"+nodeId +"_" + index,
								index : index,
								html : "",
								color : "blue",
								regions : [{
									start : biojsData.opt.selection.start,
									end : biojsData.opt.selection.end,
	
								}]
							});
	
							/* re-apply css for coverageFileSelected */
							var x = 0;
							while(cy.$("#n"+x) == undefined){
									x++;
							}
							var refNode = cy.$("#n"+x);
							$(".pane-south-data-value[file="+refNode.data("coverageFileSelected")+"]").removeClass("unselected").addClass("selected");
		    				$(".pane-south-data-value[file]").not("[file="+refNode.data("coverageFileSelected")+"]").removeClass("selected").addClass("unselected");
	
							biojsData.onAnnotationClicked(function(e) {

								//var id = e.index;
								var target = e.target;
								var id = target.attr("index");
								var classId = nodeId +"_" + id;
								var $className = $("."+target.attr("name"));

								if ($("#annotationpicker_"+classId).get(0) == undefined) {
									$className.parent().append('<div class="color-chooser annotation-chooser" id="annotationpicker_' + classId + '"style="position:absolute;display:none;"></div>');
								}
								
								//var id = parseInt($(".annotation." + index).attr("index"));
								$('#annotationpicker_' + classId).picker({
									onShow : function(colpkr) {

									},
									onChangeColor : function(col) {
										var colorSelected = "#"+$.picker.hsbToHex(col);
										biojsData._annotations[id].color = colorSelected;
										$className.attr("color", colorSelected).css("border-color", colorSelected);
										$('.annotation-chooser').css("backgroundColor", colorSelected);
									},
									onChangeName : function() {
										biojsData._annotations[id].name = $(this).data("picker").name;
										$className.attr("name", biojsData._annotations[id].name);
									},
									onChangeText : function() {
										biojsData._annotations[id].html = $(this).data("picker").text;
									},
									onHide : function(colpkr) {
										$('#annotationpicker_' + classId).hide();
									},
									onSubmit : function (col) {
										var colorSelected = "#"+$.picker.hsbToHex(col);
										biojsData._annotations[id].color = colorSelected;
										$className.attr("color", colorSelected).css("border-color", colorSelected);
										$('.annotation-chooser').css("backgroundColor", colorSelected);
										biojsData._annotations[id].name = $(".sequencepicker").data("picker").name;
										biojsData._annotations[id].html = $(".sequencepicker").data("picker").text;
										$('#annotationpicker_' + classId).hide();
									}
									
								});
	
								var color = $.Color(target.attr("color")).toHexString();
								var top = target.position().top - 10;
								var left = $className.position().left;
	
								
								$('#annotationpicker_' + classId).css({
									top : top,
									left : left,
									backgroundColor : color,
									display : "block"
								}).pickerSetValues(color, 1, undefined, biojsData._annotations[id].name, biojsData._annotations[id].html).pickerShow();
	
							});
							biojsData.setSelection(0, 0);
							
						});
						
						if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
	
					}
				}, {
					name : "Hightlight",
					attr : {
						id : "addHightlight"
					},
					selecton : function() {
						//highlight the characters within the position x to y, included.
						var $biojs = $(".ui-sequences");	
						$biojs.each(function() {
							//var idNodes = this.data("id");
							var biojsData = $(this).data("biojsData");
							var index;						
							var nbHighlights = biojsData.opt.highlights.length;
							//index need to be define here to keep the right value
							for(index=0; index<=nbHighlights; index++){
								if(!biojsData.checkHighlightId("id_" + index)){
									break;
								};
							}
							
							biojsData.addHighlight({
								"start" : biojsData.opt.selection.start,
								"end" : biojsData.opt.selection.end,
								"color" : "white",
								"background" : "red",
								"id" : "id_" + index
							});
							biojsData.setSelection(0, 0);
						});
						
						if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					}
				}]
			}, {
				/*Remove annotation and highlight menu*/
				name : "Remove",
				attr : {
					id : "removeSeqElements"
				},
				open : function() {
	
				},
				items : [{
					name : "All",
					attr : {
						id : "removeAll"
					},
					selecton : function() {
						var $biojs = $(".ui-sequences-visible");
						$biojs.each(function() {
							var biojsData = $(this).data("biojsData");
							biojsData.removeAllAnnotations();
							biojsData.unHighlightAll();
						});
						
						if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					}
				}, {
					name : "All Annotation",
					attr : {
						id : "removeAllAnnotation"
					},
					selecton : function() {
						var $biojs = $(".ui-sequences-visible");
						$biojs.each(function() {
							var biojsData = $(this).data("biojsData");
							biojsData.removeAllAnnotations();
						});
						
						if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					}
				}, {
					name : "All Hightlights",
					attr : {
						id : "removeAllHightlights"
					},
					selecton : function() {
						var $biojs = $(".ui-sequences-visible");
						$biojs.each(function() {
							var biojsData = $(this).data("biojsData");
							biojsData.removeAllHighlights();
						});
						
						if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					}
				},{
					name : "Hightlight",
					attr : {
						id : "removeHightlight"
					},
					selecton : function() {
						var $biojs = $(".ui-sequences-visible");
						$biojs.each(function() {
							var biojsData = $(this).data("biojsData");
							biojsData.removeHighlight(undefined, biojsData.opt.selection.start, biojsData.opt.selection.end);
						});
						
						if($(".menu-button-south").hasClass("active")){
								$(".menu-button-south").removeClass("active");
					      	 	$(".flexnav-show").removeClass("flexnav-show");
						};
					}
				}]
			},{
				/*lock button*/
				name : "Hold",
				checkable : true,
				attr : {
					id : "holdSouth"
				},
				selecton : function() {
					$("body").layout().disableClosable("south");
					$("body").layout().disableSlidable("south");
					
					if($(".menu-button-south").hasClass("active")){
						$(".menu-button-south").removeClass("active");
					     $(".flexnav-show").removeClass("flexnav-show");
					};
				},
				selectoff : function() {
					$("body").layout().enableClosable("south");
					$("body").layout().close("south");
					$(window).resize(); // trigger scrollBarSupport() for dataTable size
					
					if($(".menu-button-south").hasClass("active")){
						$(".menu-button-south").removeClass("active");
					     $(".flexnav-show").removeClass("flexnav-show");
					};
				}
			}]
		}; 
		this.menuSouth =menuSouth;
	}
					
	CytoscapeMenu.prototype = {
	
		/**
		 * When body is ready, create the menu north with cytoscape graph functions.
		 *
		 * @method menuGraph
		 * @param {object} config parsed configuration file
		 */
		menuGraph : function(config){
			
			if(config.with_themes){
				menuNorth.items[2].items.push(subMenuThemes);
			}
			
			this.$el.menubar(this.menuNorth)
				/* Display name file in menu */
				.children().append('<form class="ui-state-default ui-corner-all ui-menu-title" id="files_upload">\
										<span>\
											<label> Current File : </label>\
											<input type="text" id="selectedfile" class="ui-menu-title ui-state-default value="" readonly" />\
										</span>\
										<span>\
											<label> Session : </label>\
											<input type="text" id="currentSession" class="ui-menu-title ui-state-default" value="" readonly" />\
										</span>\
									 </form>');
				
			
			var fileName = localStorage.getItem("fileName");
			$('#selectedfile').val(fileName);
			
	},
				
		/**
		* When body is ready, create the menu north with cytoscape graph functions.
		* @method menuSeq
		*/
		menuSeq : function(){
			this.$el.menubar(this.menuSouth);
			
		},
	 
		/**
		* Save the current css style of the graph elements in json files.
		* This function is not activated.
		* @beta
		* @method saveStyle
		*/
		//TODO: Optimize css save with the new function cy.style() of cytoscape.js
		//TODO: save file with tjson extension
		/*saveStyle : function($el){
			var elements = cy.elements();
			var nodes = cy.nodes().css();
			var edges = cy.edges().css();
			var core =  {"background-color" : $.Color($("#cy").css('background-color')).toHexString()};
			var core_selected = cy.style()._private.coreStyle;
			elements.select();
			var nodes_selected = cy.nodes(":selected").css();
			var edges_selected = cy.edges(":selected").css();
			elements.unselect();
			$('body').layout().close("south");

			var css_save = {
				"core": {
	       	 		"style": {
	            		"unselected": core, 
	            		"selected": core_selected,
	        		}
	    		},
	   		 	"nodes": {
	        		"style": {
	            		"unselected": nodes, 
	            		"selected": nodes_selected
	       			}
	    		},
	    		"edges": {
	        		"style": {
	            		"unselected": edges,
	            		"selected":  edges_selected
	        		}
	   			 }
			};
		
			var text_save = JSON.stringify(css_save);

			uri_content = "data:application/jst;filename=style.jst;charset=utf-8," + encodeURIComponent(text_save);    
	    	$.attr("href", uri_content);
	   
		},*/
		
		/**
		* Restore or load css style for graph elements with tjson files or files save with saveStyle() function.
		* This function is not activated.
		* @beta
		* @method applyStyle
		*/
		//TODO: Adapt this function to new tjson format
		//TODO: save with blob and saveAs
		/*applyStyle: function(jsonStyle){
			cy.elements(":selected").unselect();
			
			var rootId = cy._private.root.data("id");
			var root = cy.nodes("#"+rootId);
			var nodes = cy.nodes();
			var edges = cy.edges();
			
			var curStyleObj = cy.style(),
	            elementCount = jsonStyle.length;
			
			cy.style().resetToDefault(); //reset style to default css style
			
			for (var i = 0; i < elementCount; i++) { //Support root style
				
				if (jsonStyle[i].selector == "core") {
					$("#cy").css("background-color", jsonStyle[i].css.backgroundColor);
				}
			
				if (jsonStyle[i].selector == "root:unselected") {
					jsonStyle[i].selector = "#" + rootId + ":unselected";
				} else if (jsonStyle[i].selector == "root:selected") {
					jsonStyle[i].selector = "#" + rootId + ":selected";
				} else if (jsonStyle[i].selector == "root") {
					jsonStyle[i].selector = "#" + rootId;
				}
	
		
			}
			
			cy.style().fromJson(jsonStyle).update();//re-apply hide rr edges and content definition
				
			var edges_ff = [],
				edges_rr = [];
				
			edges.each(function() {
				this.css("content", "data(direction)");
				if (this.data("direction") == "RR" || this.data("direction") == "rr") {
					edges_rr.push(this);
					this.hide();
				} else if ((this.data("direction") == "rf" || this.data("direction") == "RF") && this.data("source") < this.data("target")) {
					edges_rr.push(this);
					this.hide();
				}

			}); 

			nodes.css("content", "data(id)");
	
		},*/
		
		/**
		* Save the current session:
		* - Vizmapper edges and nodes 
		* - Starter and substarter selected 
		* - File coverage selected
		* - Nodes positions
		*
		* @method saveSession
		* @private
		*/
		saveSession : function() {
			var sessionSave = {}; // { "nodes" : {}, "edges": {}, "position":[] };
			var saveNodes;
			var saveEdges;
			var nbNodes= cy.nodes().length;
			var currentTab = $("#side").find(".ui-state-active").children();
			
			var jsonGraph = { graph: JSON.parse(localStorage.getItem("graphData"))}; //save the graph structure 
			
			var x = 0;
			while(cy.$("#n"+x) == undefined){
					x++;
			}
			var headerSessionSave = { //define initial parameters (json name, starter and substarter selected, coverage file selected) 
				header : {
					jsonFileName : localStorage.getItem("fileName"),
					starterSelected : localStorage.getItem("starterSelected"),
					substarterSelected : localStorage.getItem("substarterSelected"),
					coverageFilesSelected : cy.$('#n'+x).data("coverageFileSelected")
				}
			}; 

			/* Force stop propagation */ 
			//TODO: fix bug with e.stopImmediatelyPropagation() in initVisu.js and delete this
			$("#vizmapper_link").click(function(e){
	   			e.stopImmediatePropagation();
			});
			$("#vizstyler_link").click(function(e){
	   			e.stopImmediatePropagation();
			});
			
			/*Save nodes vizmapper */
			$("#vizmapper_link").click();
			$("#VizmapperNodes_link").click();
			saveNodes = $("#vizmapper.content").vizMapSave();
			sessionSave.nodes = saveNodes;
			
			/*Save edges vizmapper */
			$("#VizmapperEdges_link").click();
			saveEdges = $("#vizmapper.content").vizMapSave();
			sessionSave.edges = saveEdges;
			
			/* Restore the current tab after save*/
			if(currentTab[0]==$("#vizstyle_link")[0]){
				$("#vizstyle_link").click();
				$("#VizstyleCore_link").click();
			}else if(currentTab[0]==$("#vizmapper_link")[0] && currentTab[1]==$("#VizmapperEdges_link")[0]) {
				$("#vizmapper_link").click();
				$("#VizmapperEdges_link").click();
			}else if(currentTab[0]==$("#vizmapper_link")[0] && currentTab[1]==$("#VizmapperNodes_link")[0]){
				$("#vizmapper_link").click();
				$("#VizmapperNodes_link").click();
			};
		
			
			var position = [];
			//for(var i=0; i< nbNodes; i++) {
			var i =0;
			var j =0;
			var biojs = [];
			cy.nodes().each(function(){
					var nodePosition = this.position(); //select node
					var nodebiojsData = this.data("biojsData");
					
					if(cy._private.root != undefined){
						var rootId = cy._private.root.data("id");
						if(nodePosition == null){
							nodePosition = cy.$("#"+rootId).position(); //select node
						}
					}
					position[i] = nodePosition;
					
					
					if(nodebiojsData!=undefined){
						biojs.push({ 
								"id":nodebiojsData.opt.id,
								"format": nodebiojsData.opt.format,
								"highlights": nodebiojsData.opt.highlights,
								"annotations": nodebiojsData.opt.annotations
						});
					};
					
					i++;
	
			});
			
			sessionSave.position = position;
			sessionSave.biojs = biojs;
			$.extend(sessionSave, headerSessionSave, jsonGraph); //merge vizmappers save, header and graph structure 
		
			/* Save and download sjson file*/
			var textSessionSave = JSON.stringify(sessionSave);
			var blob = new Blob([textSessionSave], {type: "text/plain;charset=utf-8"});
			saveAs(blob, "session.sjson");
			
		},
		
		/**
		* Save the current vizmapper configuration:
		* - Vizmapper edges and nodes 
		*
		* @method saveVizmapper
		* @private
		*/
		saveVizmapper : function() {
			var vizSave = {}; // { "nodes" : {}, "edges": {}, "position":[] };
			var saveNodes;
			var saveEdges;
			var nbNodes= cy.nodes().length;
			var currentTab = $("#side").find(".ui-state-active").children();

			/* Force stop propagation */ 
			//TODO: fix bug with e.stopImmediatelyPropagation() in initVisu.js and delete this
			$("#vizmapper_link").click(function(e){
	   			e.stopImmediatePropagation();
			});
			$("#vizstyler_link").click(function(e){
	   			e.stopImmediatePropagation();
			});
			
			/*Save nodes vizmapper */
			$("#vizmapper_link").click();
			$("#VizmapperNodes_link").click();
			saveNodes = $("#vizmapper.content").vizMapSave();
			vizSave.nodes = saveNodes;
			
			/*Save edges vizmapper */
			$("#VizmapperEdges_link").click();
			saveEdges = $("#vizmapper.content").vizMapSave();
			vizSave.edges = saveEdges;
			
			/* Restore the current tab after save*/
			if(currentTab[0]==$("#vizstyle_link")[0]){
				$("#vizstyle_link").click();
				$("#VizstyleCore_link").click();
			}else if(currentTab[0]==$("#vizmapper_link")[0] && currentTab[1]==$("#VizmapperEdges_link")[0]) {
				$("#vizmapper_link").click();
				$("#VizmapperEdges_link").click();
			}else if(currentTab[0]==$("#vizmapper_link")[0] && currentTab[1]==$("#VizmapperNodes_link")[0]){
				$("#vizmapper_link").click();
				$("#VizmapperNodes_link").click();
			};
		
			/* Save and download sjson file*/
			var textSessionSave = JSON.stringify(vizSave);
			var blob = new Blob([textSessionSave], {type: "text/plain;charset=utf-8"});
			saveAs(blob, "vizmapConf.vjson");
			
		},
		
		
		
		/**
		* Restore previous session (sjson file):
		* - Vizmapper edges and nodes
		* - Starter and substarter selected
		* - File coverage selected 
		* - Nodes positions
		* 
		* @method loadSession
		* @param {FilesList} files Uploaded file
		* @private
		* 
		*/
		//TODO: implement updateInitialOpts
		loadSession : function(files) {   	
			if (!files.length) {
				alert('Please select a file!');
			}
			 
			var self = this,
				file = files[0],
				extensionSJSON = (/\.(sjson)$/i).test(file.name),
				extensionVJSON = (/\.(vjson)$/i).test(file.name),			    		    
				reader = new FileReader();
				    
			reader.onloadend = function(event) {
				var target = event.target;

				if (target.readyState == reader.DONE) {// DONE == 2
					if (extensionSJSON) { //check if it's a sjson file'
					
					var sessionData = target.result;
					localStorage.setItem('sessionData', sessionData);
					
					var sessionDataParse = JSON.parse(sessionData);
					var header = sessionDataParse.header;
					var jsonfileName =  header.jsonFileName;
					var jsonGraph = sessionDataParse.graph;
					localStorage.setItem("fileName", jsonfileName);
					$('#selectedfile').val(jsonfileName);

						//graphData = buildGraph(starter,substarter, header.coverageFilesSelected);
						localStorage.setItem('graphData', JSON.stringify(jsonGraph));
						window.location.href = 'visu_graph.html';
						
						return jsonGraph;
					} else  if(extensionVJSON){
						//TODO: Do vjson implementation with this 
						var vizData = JSON.parse(target.result);
						var currentTab = $("#side").find(".ui-tabs-selected").children();
						
						var nodes = cy.nodes();
						var edges = cy.edges();
						
						$("#vizmapper_link").click();
						
						if(cy.nodes().data("coverage") == undefined){
							delete vizData.edges;
							var vizDataLen =  vizData.nodes.mappers.length;
							for(var i=0; i< vizDataLen; i++){
								
								var attrCssLen = vizData.nodes.mappers[i].opts.attrCss.length;
								for( var j =0; j< attrCssLen; j++){
									if(vizData.nodes.mappers[i].opts.attrCss[j] == "colors"){
										vizData.nodes.mappers[i].opts.attrCss.splice(j);
									}
									
								}
								
								var propertyLen = vizData.nodes.mappers[i].opts.property.length;
								for( var k =0; k< propertyLen; k++){
									if(vizData.nodes.mappers[i].opts.property[k] == "Coverage"){
										vizData.nodes.mappers[i].opts.property.splice(k);
									}
									
								}
								
								if(vizData.nodes.mappers[i].property == "coverage"){
									vizData.nodes.mappers.splice(i);
								}
							}
							
						}else{
							$("#VizmapperEdges_link").click();
							$("#vizmapper.content").vizMapRestore(vizData.edges, self.currentConfig);
						}
						$("#VizmapperNodes_link").click();

						$("#vizmapper.content").vizMapRestore(vizData.nodes, self.currentConfig);

						// Restore the current tab after save
						if (currentTab.get(0) == $("#vizstyle_link").get(0) ) {
							$("#vizstyle_link").click();
							$("#core_link").click();
						} else if (currentTab.get(0)  == $("#vizmapper_link").get(0)  && currentTab.get(1)  == $("#VizmapperEdges_link").get(0) ) {
							$("#vizmapper_link").click();
							$("#VizmapperEdges_link").click();
						}
						
						if($('#edgesLabel').children(":first").hasClass("ui-menu-checked")){
							edges.css("content", "data(direction)");
						}else{
							edges.css("content", "");
						}
						
						if($('#nodesLabel').children(":first").hasClass("ui-menu-checked")){
							nodes.css("content", "data(id)");
						}else{
							nodes.css("content", "");
						}

					} else {
						alert('Please select SJSON, VJSON or JSON files');
					}
				};

			};
			
			reader.onerror = function(file) {
				alert('Error on loading file');
			};

			reader.onabort = function(file) {
				alert('Loading is aborted');
			}; 
  
		    reader.readAsText(file);
 	
    	},
    };	
	
	/**
	 * Show/hide labels of graph elements (nodes and edges)
	 * @class labelsToggle
	 * @param {string} type type of element (node or edges)
	 * @param {boolean} bool boolean value : 1 for show, 0 for hide  
	 * @constructor
	 */
	
	function labelsToggle (){
		
	}
	
	labelsToggle.prototype = {
		/**
		* Show/hide edges labels
		* @param {boolean} bool 0 labels are hidden, 1 labels are showed (default: 1)
		* @method labelEdgesToggle
		* @private
		*/
	   labelEdgesToggle : function(bool) {

			if (!bool) {
				cy.edges().css("content", "");
				cy.edges(":selected").each(function(){
					this.data().previousCss.content = "";
				});
				$(".label.edges > input").attr("disabled", true);
				$(".label.edges > select").attr("disabled", true);
				$(".ui-continuous_map[property=edges_label]").hide();
				

			} else {
				cy.edges().css("content", "data(direction)");
				cy.edges(":selected").each(function(){
					this.data().previousCss.content =  "data(direction)";
				});
				$(".label.edge > input").attr("disabled", false);
				$(".label.edge > select").attr("disabled", false);
				$(".ui-continuous_map[property=edges_label]").show();
			}
	
		},
		/**
		* Show/hide nodes labels
		* @param {boolean} bool 0 labels are hidden, 1 labels are showed (default: 1)
		* @method labelNodesToggle
		* @private
		*/
		labelNodesToggle: function(bool) {
			if (!bool) {
				cy.nodes().css("content", "");
				cy.nodes(":selected").each(function(){
					this.data().previousCss.content = "";
				});
				$(".label.nodes > input").attr("disabled", true);
				$(".label.nodes > select").attr("disabled", true);
				$(".ui-continuous_map[property=nodes_label]").hide();
			} else {
				cy.nodes().css("content", "data(id)");
				cy.nodes(":selected").each(function(){
					this.data().previousCss.content = "data(id)";
				});
				$(".label.nodes > input").attr("disabled", false);
				$(".label.nodes > select").attr("disabled", false);
				$(".ui-continuous_map[property=nodes_label]").show();
			}
		}	
		
	};	
	
    
	var methods={
		/**
		 * Initialize and display menu
		 * @method init
		 * @param type the type of menu.
		 * @param config  the config file parsed. 
		 * @for CytoscapeMenu
		 */
		init : function(type, config){
			var $el= $(this);	
			var cytoscapeMenu = new CytoscapeMenu($el, config);
			$el.data("CytoscapeMenu", cytoscapeMenu);
			
			if(type == "graph"){
				cytoscapeMenu.menuGraph(config);
				$el.children(":first").addClass('flexnav one-page').attr("data-breakpoint", 1140);
				
				$el.children(":first").flexNav({
					'id' : 'north',
					'buttonSelector' : '.menu-button-north',
				}); 

			}else if(type == "sequence"){
				cytoscapeMenu.menuSeq();
				$el.children(":first").addClass('flexnav one-page').attr("data-breakpoint", 1140);
				
				$el.children(":first").flexNav({
					'id' : 'south',
					'buttonSelector' : '.menu-button-south',
				}); 

			}
			
			
			
			
		return cytoscapeMenu;
		},
		/*toggle: function(type, bool){
			if(type == "nodes"){
				labelsToggle.labelNodesToggle(bool);
			}else if(type == "edges"){
				labelsToggle.labelEdgesToggle(bool);
			}				
		}*/
	};
	
	/** extend jquery functions
	 * 
	 * @namespace
	 */
	$.fn.extend({
			cytoscapeMenu: methods.init,
	});
})(jQuery);