/**
 * @summary     cytoscape.picker
 * @description Display a bubble pickers/setters for color, shape , opacity and size. 
 * @version     
 * @file        cytoscape.picker.js
 * @author      Alexan Andrieux
 * @contact     <a href="mailto:alexan.andrieux@inria.fr"> Alexan Andriux</a>
 * @category
 *
 * @copyright 
 *
 * This source file is free software, under AGLP License
 * 
 * This source file is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 * 
 * For details please refer to: <a href="http://www.gnu.org/licenses/agpl-3.0.html"></a>
 */


/**
 * Based on Jose Vargas colpick Color Picker <a href="colpick.com/plugin">colpick.com/plugin</a> licensed under GPL license and Stefan Petre's Color Picker <a href="www.eyecon.ro">www.eyecon.ro</a>, dual licensed under the MIT and GPL licenses
 * 
 * It's a manager for all picker/setter bubble. His required for cytoscape.vizmapper and menu 2 in cytoscape.menu.
 * Cytoscape picker allow to display a bubble with a picker/setter for :
 * - color
 * - shape
 * - size
 * - annotation name
 * - annotation text
 * 
 * @module picker
 * 
 * @requires <a href='http://blog.jquery.com/2011/09/12/jquery-1-6-4-released/'>jQuery 1.6.4+</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-1.4.2.min.js"></script>
 * 
 * 
 * @example
 * 	//Color chooser
 * 	$(".color-chooser .contunuous_map").picker({
 * 		onShow: function () {
 *				...
 *		},
 *		onChangeColor: function(){
 *				...
 *		},
 *		onHide: function () {
 *				...
 *		},
 *		onSubmit : function () {
 *				...
 *		},
 * 	});
 *
 * 	//Shape chooser
 * 	$(".shape-chooser .contunuous_map").picker({
 * 		onShow: function () {
 *				...
 *		},
 *
 *		onChangeShape: function(){
 *				...
 *		},
 * 		onChangeSize: function(){
 *				...
 *		},
 *		onHide: function () {
 *				...
 *		},
 * 		onSubmit : function () {
 *				...
 *		},
 * 	});
 *
 * 	//Commentaries chooser
 *	$(".shape-chooser .contunuous_map").picker({
 * 		onShow: function () {
 *				...
 *		},
 *
 *		onChangeColor: function(){
 *				...
 *		},
 * 		onChangeName: function(){
 *				...
 *		},
 * 		onChangeText: function(){
 *				...
 *		},
 *		onHide: function () {
 *				...
 *		},
 *		onSubmit : function () {
 *				...
 *		},
 * 	});
 * 	//Advanced chooser
 * 	$(".color-chooser .shape-chooser .contunuous_map").picker({
 * 		onShow: function () {
 *				...
 *		},
 *
 *		onChangeShape: function(){
 *				...
 *		},
 * 		onChangeSize: function(){
 *				...
 *		},
 *		onHide: function () {
 *				...
 *		},
 *		onSubmit : function () {
 *				...
 *		},
 * 	}); 
 */

(function ($) {
"use strict";
	/**
	 * Create a new instance of pickers/setters.
	 * The type of bubble pickers/setters is define by classes of the selector:
	 * - color-chooser : just the color picker
	 * - shape-chooser : just the shape picker without size input
	 * - color-chooser continuous_map: just the color picker
	 * - shape-chooser continuous_map: the shape picker with size input  
	 * - color-chooser annotation-chooser : the color picker with commentaries input for name and text area
	 * - color-chooser shape-chooser continuous_map advanced : advanced picker with color picker, shape picker and size input
	 * @class picker
	 * @param {object} defaults the object contented events and defaults values of attributes.
	 * @constructor
	 * 
	 */
	var picker = function () {
		var
			/**
			 * Create template for color picker.
			 * @property tpl
			 * @type String
			 * @private
			 */
			tpl = '<div class="picker colorpicker">\
					<div class="colorpicker_color">\
						<div class="picker_color">\
							<div class="picker_color_overlay1">\
								<div class="picker_color_overlay2">\
									<div class="picker_selector_outer">\
										<div class="picker_selector_inner">\
										</div>\
									</div>\
								</div>\
							</div>\
						</div>\
					<div class="picker_hue">\
						<div class="picker_hue_arrs">\
							<div class="picker_hue_larr">\
							</div>\
							<div class="picker_hue_rarr">\
							</div>\
						</div>\
					</div>\
					<div class="picker_new_color">\
					</div>\
					<div class="picker_current_color">\
					</div>\
					<div class="picker_hex_field">\
						<div class="picker_field_letter">#\
						</div>\
						<input type="text" maxlength="6" size="6" />\
					</div>\
					<div class="picker_rgb_r picker_field_color">\
						<div class="picker_field_letter">R\
						</div>\
						<input type="text" maxlength="3" size="3" />\
						<div class="picker_field_arrs">\
							<div class="picker_field_uarr">\
							</div>\
							<div class="picker_field_darr">\
							</div>\
						</div>\
					</div>\
					<div class="picker_rgb_g picker_field_color">\
						<div class="picker_field_letter">G\
						</div>\
						<input type="text" maxlength="3" size="3" />\
						<div class="picker_field_arrs">\
							<div class="picker_field_uarr">\
							</div>\
							<div class="picker_field_darr">\
							</div>\
						</div>\
					</div>\
					<div class="picker_rgb_b picker_field_color">\
						<div class="picker_field_letter">B\
						</div>\
						<input type="text" maxlength="3" size="3" />\
						<div class="picker_field_arrs">\
							<div class="picker_field_uarr">\
							</div>\
							<div class="picker_field_darr">\
							</div>\
						</div>\
					</div>\
					<div class="picker_hsb_h picker_field_color">\
						<div class="picker_field_letter">H\
						</div>\
						<input type="text" maxlength="3" size="3" />\
						<div class="picker_field_arrs">\
							<div class="picker_field_uarr">\
							</div>\
							<div class="picker_field_darr">\
							</div>\
						</div>\
					</div>\
					<div class="picker_hsb_s picker_field_color">\
						<div class="picker_field_letter">S\
						</div>\
						<input type="text" maxlength="3" size="3" />\
						<div class="picker_field_arrs">\
							<div class="picker_field_uarr">\
							</div>\
							<div class="picker_field_darr">\
							</div>\
						</div>\
					</div>\
					<div class="picker_hsb_b picker_field_color">\
						<div class="picker_field_letter">B\
						</div>\
						<input type="text" maxlength="3" size="3" />\
						<div class="picker_field_arrs">\
							<div class="picker_field_uarr">\
							</div>\
							<div class="picker_field_darr">\
							</div>\
						</div>\
					</div>\
					<div class="picker_opacity picker_field_color">\
						<div class="picker_field_letter">%\
						</div>\
						<input type="text" maxlength="3" size="3" />\
						<div class="picker_field_arrs">\
							<div class="picker_field_uarr">\
							</div>\
							<div class="picker_field_darr">\
							</div>\
						</div>\
					</div>\
					<div class="picker_submit">\
					</div>\
				</div>\
				</div>',
			
			/**
			 * Create template for shape picker. 
			 * @property tps
			 * @type String
			 * @private
			 */
			tps = '<div class="picker shapepicker"> </div>',
			
			/**
			 * Extension of color picker template  for nodes with advanced vizmapper/
			 * @property node_shapepicker
			 * @type String
			 * @private
			 */
			node_shapepicker = '<div id="node_shapepicker" class="shapepicker_shape"><label class="style"> Nodes Shape: </label></div>',
			/**
			 * Extension of color picker template  for edges with advanced vizmapper.
			 * @property line_shapepicker
			 * @type String
			 * @private
			 */
			line_shapepicker = '<div id="line_shapepicker" class="shapepicker_shape"><label class="style">  Border Style: </label></div>',
			/**
			 * Extension of color picker template  for edges with advanced vizmapper.
			 * @property arrow_shapepicker
			 * @type String
			 * @private
			 */
			arrow_shapepicker = '<div id="arrow_shapepicker" class="shapepicker_shape"><label class="style"> Arrow Style: </label></div>',
			/**
			 * Extension of color picker template  for annotation with menu 2 of cytoscape menu.
			 * @property node_sequencepicker
			 * @type String
			 * @private
			 */
			node_sequencepicker = '<div id="node_sequencepicker" class="picker_sequence picker_field_sequence">\
										<div class="picker_field_letter">Name \
										</div>\
										<input style="position:relative;" type="text" maxlength="30" size="30"/>\
										<label class="style" style="position:relative;">  Commentary : </label>\
										<textarea  style="position:relative;" type="text"> </textarea>\
									</div>',
			
			/**
			 * Extension of shape picker template. 
			 * @property size_shapepicker 
			 * @type String
			 * @private
			 */
			size_shapepicker = '<div class="shapepicker_size">\
									<div class="picker_size picker_field_size">\
										<div class="picker_field_letter">Size \
										</div>\
										<input type="text" maxlength="3" size="3" />\
										<div class="picker_field_arrs">\
											<div class="picker_field_uarr">\
											</div>\
											<div class="picker_field_darr">\
											</div>\
										</div>\
									</div>\
								</div>',
			
			/**
			 * Initialize defaults attributes.
			 * @property defaults 
			 * @type object
			 */
			defaults = {
				/**
				 * Define type of event to show bubble pickers/setters
				 * @attribute showEvent
				 * @type String 
				 * @default click
				 */
				showEvent: 'click',
				/**
				 * On show event
				 * @event onShow
				 */
				onShow: function () {},
				/**
				 * On change color event
				 * @event onChangeColor
				 */
				onChangeColor: function () {},
				/**
				 * On change size event
				 * @event onChangeSize
				 */
				onChangeSize: function () {},
				/**
				 * On change name of a commentary event
				 * @event onChangeName
				 */
				onChangeName: function () {},
				/**
				 * On change content of a commentary event
				 * @event onChangeText
				 */
				onChangeText: function () {},
				/**
				 * On change shape event
				 * @event onChangeShape
				 */
				onChangeShape: function () {},
				/**
				 * On focus on input event
				 * @event onFocus
				 */
				onFocus: function () {},
				/**
				 * On blur on input event
				 * @event onBlur
				 */
				onBlur: function () {},
				/**
				 * Before show bubble pickers/setters event
				 * @event onBeforeShow
				 */
				onBeforeShow: function(){},
				/**
				 * On hide bubble pickers/setters event
				 * @event onHide
				 */
				onHide: function () {},
				/**
				 * On click OK button event
				 * @event onSubmit
				 */
				onSubmit: function () {},
				/**
				 * Define the theme (light/dark)
				 * @attribute colorScheme
				 * @type String 
				 * @default light
				 */
				colorScheme: 'light',
				/**
				 * Define the default color
				 * @attribute color
				 * @type String 
				 * @default 3289c7
				 */
				color: '3289c7',
				/**
				 * Define the default opacity
				 * @attribute opacity
				 * @type int
				 * @default 1
				 */
				opacity: 1,
				/**
				 * Define the default shape
				 * @attribute shape
				 * @type string
				 * @default ""
				 */
				shape: "",
				/**
				 * Define the default behavior of bubble pickers/setters
				 * @attribute livePreview
				 * @type boolean
				 * @default true
				 */
				livePreview: true,
				/**
				 * Define the default behavior of bubble pickers/setters
				 * @attribute flat
				 * @type boolean
				 * @default false
				 */
				flat: false,
				/**
				 * Define the type of color picker
				 * @attribute layout
				 * @type String
				 * @default full
				 */
				layout: 'full',
				/**
				 * Define display submit button
				 * @attribute submit
				 * @type int
				 * @default 1
				 */
				submit: 1,
				/**
				 * Define text of submit button
				 * @attribute submitText
				 * @type String
				 * @default OK
				 */
				submitText: 'OK',
				/**
				 * Define the height for color sliders
				 * @attribute height
				 * @type int
				 * @default 156
				 */
				height: 156
			},
			
			/**
			 * Fill the RGB inputs.
			 * @method fillRGBFields
			 * @private 
			 */
			fillRGBFields = function  (hsb, cal) {
				var rgb = hsbToRgb(hsb);
				$(cal).data('picker').fields
					.eq(1).val(rgb.r).end()
					.eq(2).val(rgb.g).end()
					.eq(3).val(rgb.b).end();
			},
			/**
			 * Fill the HSB inputs.
			 * @method fillHSBFields
			 * @private 
			 */
			fillHSBFields = function  (hsb, cal) {
				$(cal).data('picker').fields
					.eq(4).val(Math.round(hsb.h)).end()
					.eq(5).val(Math.round(hsb.s)).end()
					.eq(6).val(Math.round(hsb.b)).end();
			},
			/**
			 * Fill the Hex input.
			 * @method fillHexFields
			 * @private 
			 */
			fillHexFields = function (hsb, cal) {
				$(cal).data('picker').fields
					.eq(0).val(hsbToHex(hsb));
			},
			/**
			 * Fill the size input for advanced vizmapper.
			 * @method fillSizeFields
			 * @private 
			 */
			fillSizeFields = function (sz, cal) {
				$(cal).data('picker').fields
					.eq(8).val(sz).end();

			},
			/**
			 * Fill the opacity input.
			 * @method fillOpacityFields
			 * @private 
			 */
			fillOpacityFields = function (op, cal) {
				$(cal).data('picker').fields
					.eq(7).val(op).end();

			},
			/**
			 * Fill the size input
			 * @method fillSizeFieldsOnly 
			 * @private 
			 */
			fillSizeFieldsOnly = function (sz, cal) {
				$(cal).data('picker').fields
					.eq(0).val(sz).end();
			},
			/**
			 * Fill the annotation input.
			 * @method fillNameFields
			 * @private 
			 */
			fillNameFields = function (nm, cal) {
				
				$(cal).data('picker').fields
					.eq(8).val(nm).end();

			},
			/**
			 * Fill the annotation area.
			 * @method fillTextFields
			 * @private 
			 */
			fillTextFields = function (txt, cal) {
				$(cal).data('picker').fields
					.eq(9).val(txt).end();
			},
			/**
			 * Set the round selector position.
			 * @method setSelectorColor
			 * @private 
			 */
			setSelectorColor = function (hsb, cal) {
				var $dataPicker = $(cal).data('picker');
				$dataPicker.selector.css('backgroundColor', '#' + hsbToHex({h: hsb.h, s: 100, b: 100}));
				$dataPicker.selectorIndic.css({
					left: parseInt($dataPicker.height * hsb.s/100, 10),
					top: parseInt($dataPicker.height * (100-hsb.b)/100, 10)
				});
			},
			/**
			 * Set the hue selector position.
			 * @method setHue
			 * @private 
			 */
			setHue = function (hsb, cal) {
				var $dataPicker = $(cal).data('picker');
				setTimeout(function(){
					$dataPicker.hue.css('top', parseInt($dataPicker.height - $dataPicker.height * hsb.h/360, 10));
				}, 20);
			},
			/**
			 * Set the current color.
			 * @method setCurrentColor
			 * @private 
			 */
			setCurrentColor = function (hsb, cal, op) {
				$(cal).data('picker').currentColor.css({
									backgroundColor : '#' + hsbToHex(hsb),
									opacity : op
				});
			},
			/**
			 * Set the new color.
			 * @method setNewColor
			 * @private 
			 */
			setNewColor = function (hsb, cal, op) {
				$(cal).data('picker').newColor.css({
									backgroundColor : '#' + hsbToHex(hsb),
									opacity : op
				});
			},
			/**
			 * On change inputs or picker in color area, 
			 * get the values:
			 * - color inputs (HSB, Hex, RGB, opacity) 
			 * - annotation inputs 
			 * - with advanced vizmapper size input
			 * This method define the event onChangeColor, onChangeName, onChangeText and for advanced vizmapper onChangeSize.
			 * @method change
			 * @private
			 */
			change = function (ev) {
				var $this = $(this),
					cal = $this.parents(".picker"),
					dataCal= cal.data('picker'),
					$dataFields = dataCal.fields,
					calGet0 = cal.get(0),
					op,
					col;
					
				if (this.parentNode.className.indexOf('_hex') > 0) {
					dataCal.color = col = hexToHsb(fixHex(this.value));
					fillRGBFields(col, cal.get(0));
					fillHSBFields(col, cal.get(0));
					
				} else if (this.parentNode.className.indexOf('_hsb') > 0) {
					dataCal.color = col = fixHSB({
						h: parseInt(cal.data('picker').fields.eq(4).val(), 10),
						s: parseInt(cal.data('picker').fields.eq(5).val(), 10),
						b: parseInt(cal.data('picker').fields.eq(6).val(), 10)
					});
					
					dataCal.opacity = op = fixOpacity(parseInt(cal.data('picker').fields.eq(7).val(), 10));
					
					fillRGBFields(col, calGet0);
					fillHexFields(col, calGet0);
					fillOpacityFields(op, calGet0);
				} else {
					dataCal.color = col = rgbToHsb(fixRGB({
						r: parseInt(cal.data('picker').fields.eq(1).val(), 10),
						g: parseInt(cal.data('picker').fields.eq(2).val(), 10),
						b: parseInt(cal.data('picker').fields.eq(3).val(), 10)
					}));
					
					dataCal.opacity = op = fixOpacity(parseInt(cal.data('picker').fields.eq(7).val(), 10));
					
					fillHexFields(col, calGet0);
					fillHSBFields(col, calGet0);
					fillOpacityFields(op, calGet0);
				}
				
				setSelectorColor(col, calGet0);
				setHue(col, calGet0);
				setNewColor(col, calGet0, op);
				
				if (cal.hasClass("picker colorpicker shapepicker")){
					var sz;
					dataCal.size = sz = parseInt($dataFields.eq(8).val(), 10);
					
					if (ev) {
						fillSizeFields(sz,  calGet0);
					}
					
					dataCal.onChangeSize.apply(cal, $this.size);
				} else if(cal.hasClass("picker colorpicker sequencepicker")){
					var nm, txt;
					dataCal.name = nm = $dataFields.eq(8).val();
					dataCal.text = txt = $dataFields.eq(9).val();
					if (ev) {
						fillNameFields(nm,  calGet0);
						fillTextFields(txt,  calGet0);
					}
					
					dataCal.onChangeName.apply(cal, $this.name);
					dataCal.onChangeText.apply(cal, $this.text);
				};
				
				dataCal.onChangeColor.apply(cal.parent(), [col, hsbToHex(col), hsbToRgb(col), cal.data('picker').el, 0]);
			},
			/**
			 * On change input size get the value.
			 * This method define the event onChangeSize.
			 * @method changeSize
			 * @private
			 */
			changeSize = function (ev) {
				var $this = $(this),
					cal = $this.parents(".picker"),
					dataCal= cal.data('picker'),
					$dataFields = dataCal.fields,
					calGet0 = cal.get(0),
					col, op, sz;
					
					if(cal.hasClass("advanced")){
						dataCal.size = sz = parseInt($dataFields.eq(8).val(), 10);
					}else{
						dataCal.size = sz = parseInt($dataFields.eq(0).val(), 10);
					}
					
					if(isNaN(sz)){
						dataCal.size = sz = 20;
					}
					
					if (ev) {
						fillSizeFields(sz,  calGet0);
					}
					
					
					dataCal.onChangeSize.apply(cal, $this.size);
			},
			/**
			 * On click on a node shape get the name of this shape.
			 * This method define the event onChangeShape.
			 * @method changeShape
			 * @private
			 */
			changeShape = function (ev) {
				var $this = $(this);
				var cal = $this.parents(".picker");
				var $dataPicker = cal.data('picker'); 
				
				$dataPicker.shape = $this.attr("shape");
				$dataPicker.onChangeShape.apply(cal, $this.shape);
				
				cal.find("canvas").css("border-color", "#EBEBEB");
				$(this).css("border-color", "red");
				//$(cal).data('picker').selector.attr("shape", $this.attr("shape"));
			},
			/**
			 * Change style on blur. Define the blur event.
			 * @method blur
			 * @private
			 */
			blur = function (ev) {
				$(this).parents().eq(1).removeClass('picker_focus');
			},
			/**
			 * Change style on focus. Define the focus event.
			 * @method focus
			 * @private
			 */
			focus = function () {		
					$(this).parents().eq(2).data('picker').fields.parent().removeClass('picker_focus');
					$(this).parents().eq(1).addClass('picker_focus');
			},
			/**
			 * Apply change or changeSize method on key down. 
			 * @method keyDown
			 * @private
			 */
			keyDown = function (ev) {
				var cal = $(this).parents(".picker");
				if($(this).parent().hasClass("colorpicker_field")){
					var pressedKey = ev.charCode || ev.keyCode || -1;
					if ((pressedKey > charMin && pressedKey <= 90) || pressedKey == 32) {
						return false;
					}
					
					if (cal.data('picker').livePreview === true){
						change.apply(this);
					}
				}else if($(this).parent().hasClass("shapepicker_field")){
					if (cal.data('picker').livePreview === true){
							changeSize.apply(this);
					
					} 
					
				}else if($(this).parent().hasClass("sequencepicker_field")){
					if (cal.data('picker').livePreview === true){
							change.apply(this);
					}

				}
					
			},
			/**
			 * On mouse down on inputs arrows define moveIncrement on mousemove event and upIncrement on mouseup event.
			 * @method downIncrement
			 * @private
			 */
			downIncrement = function (ev) {
				ev.preventDefault ? ev.preventDefault() : ev.returnValue = false;
				var field = $(this).parent().find('input').focus();
				var current = {
					el: $(this).parent().addClass('picker_slider'),
					max: this.parentNode.className.indexOf('_hsb_h') > 0 ? 360 : (this.parentNode.className.indexOf('_hsb') > 0 ? 100 : 255),
					y: ev.pageY,
					field: field,
					val: parseInt(field.val(), 10),
					preview: $(this).parents().eq(2).data('picker').livePreview
				};
				$(document).mouseup(current, upIncrement);
				$(document).mousemove(current, moveIncrement);
			},
			/**
			 * Apply change or changeSize methods.
			 * @method moveIncrement
			 * @private
			 */
			moveIncrement = function (ev) {
				ev.data.field.val(Math.max(0, Math.min(ev.data.max, parseInt(ev.data.val - ev.pageY + ev.data.y, 10))));
				if (ev.data.el.hasClass("picker_field_color") && ev.data.preview) {
					change.apply(ev.data.field.get(0), [true]);
				}else if (ev.data.el.hasClass("picker_field_size") && ev.data.preview){
					changeSize.apply(ev.data.field.get(0), [true]);
				}
				return false;
			},
			/**
			 * Apply change or changeSize methods and off mouseup and mousemove event for inputs arrows.
			 * @method moveIncrement
			 * @private
			 */
			upIncrement = function (ev) {
				
				if (ev.data.el.hasClass("picker_field_color")) {
					change.apply(ev.data.field.get(0), [true]);
				}else if (ev.data.el.hasClass("picker_field_size")){
					changeSize.apply(ev.data.field.get(0), [true]);
				}
				
				ev.data.el.removeClass('picker_slider').find('input').focus();
				$(document).off('mouseup', upIncrement);
				$(document).off('mousemove', moveIncrement);
				return false;
			},
			
			/**
			 * On mouse down on hue slider define moveHue on mousemove event and upHue on mouseup event.
			 * @method downHue
			 * @private
			 */
			downHue = function (ev) {
				ev.preventDefault ? ev.preventDefault() : ev.returnValue = false;
				var current = {
					cal: $(this).parents().eq(1),
					y: $(this).offset().top
				};
				$(document).on('mouseup touchend',current,upHue);
				$(document).on('mousemove touchmove',current,moveHue);
				
				var pageY = ((ev.type == 'touchstart') ? ev.originalEvent.changedTouches[0].pageY : ev.pageY );
				change.apply(
					current.cal.data('picker')
					.fields.eq(4).val(parseInt(360*(current.cal.data('picker').height - (pageY - current.y))/current.cal.data('picker').height, 10))
						.get(0),
					[current.cal.data('picker').livePreview]
				);
				return false;
			},
			/**
			 * Get cursor position and apply change method.
			 * @method moveHue
			 * @private
			 */
			moveHue = function (ev) {
				var pageY = ((ev.type == 'touchmove') ? ev.originalEvent.changedTouches[0].pageY : ev.pageY );
				change.apply(
					ev.data.cal.data('picker')
					.fields.eq(4).val(parseInt(360*(ev.data.cal.data('picker').height - Math.max(0,Math.min(ev.data.cal.data('picker').height,(pageY - ev.data.y))))/ev.data.cal.data('picker').height, 10))
						.get(0),
					[ev.data.preview]
				);
				return false;
			},
			/**
			 * Fill inputs and off mouseup and mousemove event for inputs hue slider.
			 * @method moveIncrement
			 * @private
			 */
			upHue = function (ev) {
				var cal = ev.data.cal,
					dataCal = cal.data('picker'),
					calGet0 = ev.data.cal.get(0);	
				
				if (cal.hasClass("picker colorpicker")){
					fillRGBFields(dataCal.color, calGet0);
					fillHexFields(dataCal.color, calGet0);
					fillOpacityFields(dataCal.opacity, calGet0);
					//fillHSBFields(dataCal.color, calGet0);
				}else if (cal.hasClass("picker colorpicker shapepicker") ){
					fillRGBFields(dataCal.color, calGet0);
					fillHexFields(dataCal.color, calGet0);
					fillOpacityFields(dataCal.opacity, calGet0);
					fillSizeFields(dataCal.size, calGet0);
				}else if (cal.hasClass("picker shapepicker")){
					fillSizeFields(dataCal.size, calGet0);
				} else if("picker colorpicker sequencepicker"){
					fillRGBFields(dataCal.color, calGet0);
					fillHexFields(dataCal.color, calGet0);
					fillOpacityFields(dataCal.opacity, calGet0);
					fillNameFields(dataCal.name, calGet0);
					fillTextFields(dataCal.text, calGet0);
				}
				
				$(document).off('mouseup touchend',upHue);
				$(document).off('mousemove touchmove',moveHue);
				return false;
			},
			
			/**
			 * On mouse down on color selector define moveSelector on mousemove event and upSelector on mouseup event.
			 * And get the position for apply change method. 
			 * @method downSelector
			 * @private
			 */
			downSelector = function (ev) {
				ev.preventDefault ? ev.preventDefault() : ev.returnValue = false;
				var $this = $(this);
				var current = {
					cal: $this.parents().eq(1),
					pos: $this.offset()
				};
				current.preview = current.cal.data('picker').livePreview;
				
				$(document).on('mouseup touchend',current,upSelector);
				$(document).on('mousemove touchmove',current,moveSelector);

				var pageX, pageY;
				if(ev.type == 'touchstart') {
					pageX = ev.originalEvent.changedTouches[0].pageX,
					pageY = ev.originalEvent.changedTouches[0].pageY;
				} else {
					pageX = ev.pageX;
					pageY = ev.pageY;
				}

				change.apply(
					current.cal.data('picker').fields
					.eq(6).val(parseInt(100*(current.cal.data('picker').height - (pageY - current.pos.top))/current.cal.data('picker').height, 10)).end()
					.eq(5).val(parseInt(100*(pageX - current.pos.left)/current.cal.data('picker').height, 10))
					.get(0),
					[current.preview]
				);
				return false;
			},
			/**
			 * Get picker position and apply change method.
			 * @method moveSelector
			 * @private
			 */
			moveSelector = function (ev) {
				var pageX, pageY;
				if(ev.type == 'touchmove') {
					pageX = ev.originalEvent.changedTouches[0].pageX,
					pageY = ev.originalEvent.changedTouches[0].pageY;
				} else {
					pageX = ev.pageX;
					pageY = ev.pageY;
				}

				change.apply(
					ev.data.cal.data('picker').fields
					.eq(6).val(parseInt(100*(ev.data.cal.data('picker').height - Math.max(0,Math.min(ev.data.cal.data('picker').height,(pageY - ev.data.pos.top))))/ev.data.cal.data('picker').height, 10)).end()
					.eq(5).val(parseInt(100*(Math.max(0,Math.min(ev.data.cal.data('picker').height,(pageX - ev.data.pos.left))))/ev.data.cal.data('picker').height, 10))
					.get(0),
					[ev.data.preview]
				);
				return false;
			},
			/**
			 * Fill inputs and off mouseup and mousemove event for inputs hue slider.
			 * @method upSelector
			 * @private
			 */
			upSelector = function (ev) {
				var cal =  ev.data.cal,
					dataCal = cal.data('picker'),
					calGet0 = cal.get(0);
				
				if (cal.hasClass("picker colorpicker")){
					fillRGBFields(dataCal.color, calGet0);
					fillHexFields(dataCal.color, calGet0);
					fillOpacityFields(dataCal.opacity, calGet0);
					//fillHSBFields(dataCal.color, calGet0);
					
				}else if (cal.hasClass("picker colorpicker shapepicker") ){
					fillRGBFields(dataCal.color, calGet0);
					fillHexFields(dataCal.color, calGet0);
					fillOpacityFields(dataCal.opacity, calGet0);
					fillSizeFields(dataCal.size, calGet0);
				}else if (cal.hasClass("picker shapepicker")){
					fillSizeFields(dataCal.size, calGet0);
				}else if("picker colorpicker sequencepicker"){
					fillRGBFields(dataCal.color, calGet0);
					fillHexFields(dataCal.color, calGet0);
					fillOpacityFields(dataCal.opacity, calGet0);
					fillNameFields(dataCal.name, calGet0);
					fillTextFields(dataCal.text, calGet0);
				}
				
				$(document).off('mouseup touchend',upSelector);
				$(document).off('mousemove touchmove',moveSelector);
				return false;
			},
			
			/**
			 * Update picker object values.
			 * This method define the event onSubmit.
			 * @method clickSubmit
			 * @private
			 */
			clickSubmit = function (ev) {
				var cal = $(this).parents().eq(1),
					dataCal = cal.data('picker'),
					col = dataCal.color,
					op = dataCal.opacity,
					sz = dataCal.size,
					nm = dataCal.name,
					txt = dataCal.text,
					sh = dataCal.shape;
				
				if(cal.hasClass("advanced")){
					dataCal.origColor = col;
					dataCal.origOpacity = op;
					dataCal.origSize = sz;
					dataCal.origShape = sh;
					setCurrentColor(col, cal.get(0) ,op);
					dataCal.onSubmit(col, hsbToHex(col), hsbToRgb(col), dataCal.el);
				}else if(cal.hasClass("colorpicker")){
					dataCal.origColor = col;
					dataCal.origOpacity = op;
					setCurrentColor(col, cal.get(0) ,op);
					dataCal.onSubmit(col, hsbToHex(col), hsbToRgb(col), dataCal.el);
				}else if(cal.hasClass("sequencepicker")){
					dataCal.origColor = col;
					dataCal.origOpacity = op;
					dataCal.origName = nm;
					dataCal.origText = txt;
					setCurrentColor(col, cal.get(0) ,op);
					dataCal.onSubmit(col, hsbToHex(col), hsbToRgb(col), dataCal.el);
				}else if(cal.hasClass("shapepicker")){
					dataCal.origSize = sz;
					dataCal.origShape = sh;
				}
				
				
				cal.hide(ev);
				
			},
			/**
			 * Show the bubble with pickers/setters.
			 * This method define the event onShow.
			 * @method show
			 * @private
			 */
			show = function (ev) {
				// Prevent the trigger of any direct parent
				//ev.stopPropagation();

				var $this = $(this),
					pos = $this.offset(),
					viewPort = getViewport(),
					//top = pos.top
					top = pos.top + this.offsetHeight,
					left = pos.left,
					cal = $('#' + $this.data('pickerId')),
					calW = cal.outerWidth(),
					calGet0 = cal.get(0),
					calData = cal.data('picker');
					
				calData.onBeforeShow.apply(this, [calGet0]);
				
				//Apply red on correct canvas shape
				cal.find("canvas").css("border-color", "#EBEBEB");
				cal.find("canvas[shape="+cal.data('picker').shape+"]").css("border-color", "red");
				
				if (left + calW > viewPort.l + viewPort.w) {
					left -= calW;
				}else{
					left += this.offsetWidth;
				}
				
				cal.css({left: left + 'px', top: top + 'px'});
				if (cal.data('picker').onShow.apply(this, [cal.get(0)]) != false) {
					cal.show(ev);
					top = top - cal.outerHeight(); 
					left = left;
					cal.css({left: left + 'px', top: top + 'px'});
				}
				
				//Hide when user clicks outside
				$('html').mousedown({cal:cal}, hide);
				cal.mousedown(function(ev){
						ev.stopPropagation();
				});
			},
			/**
			 * Hide the bubble with pickers/setters.
			 * This method define the event onHide.
			 * @method hide
			 * @private
			 */
			hide = function (ev) {
				var evCal =ev.data.cal;
				var field = evCal.find('input').focus();
				if(field.parent().hasClass("picker_field_color")){
						change.apply(field.get(0));
				}else if(field.parent().hasClass("picker_field_size")){	
						changeSize.apply(field.get(0));
				}else if(field.parent().hasClass("picker_field_sequence")){
							change.apply(field.get(0));
				}
				
				if (evCal.data('picker').onHide.apply(this, [evCal.get(0)]) != false) {
					evCal.hide();
				}
				$('html').off('mousedown', hide);
				

			},
			/**
			 * @method getViewport
			 * @private
			 */
			getViewport = function () {
				var m = document.compatMode == 'CSS1Compat';
				return {
					l : window.pageXOffset || (m ? document.documentElement.scrollLeft : document.body.scrollLeft),
					w : window.innerWidth || (m ? document.documentElement.clientWidth : document.body.clientWidth)
				};
			},
			/**
			 * Fix the values if the user enters a negative or high value in hsb inputs
			 * @method fixHSB
			 * @private
			 */
			fixHSB = function (hsb) {
				return {
					h: Math.min(360, Math.max(0, hsb.h)),
					s: Math.min(100, Math.max(0, hsb.s)),
					b: Math.min(100, Math.max(0, hsb.b))
				};
			}, 
			/**
			 * Fix the values if the user enters a negative or high value in rgb inputs
			 * @method fixRGB
			 * @private
			 */
			fixRGB = function (rgb) {
				return {
					r: Math.min(255, Math.max(0, rgb.r)),
					g: Math.min(255, Math.max(0, rgb.g)),
					b: Math.min(255, Math.max(0, rgb.b))
				};
			},
			/**
			 * Fix the values if the user enters a wrong  value in hex inputs
			 * @method fixHex
			 * @private
			 */
			fixHex = function (hex) {
				var len = 6 - hex.length;
				if (len > 0) {
					var o = [];
					for (var i=0; i<len; i++) {
						o.push('0');
					}
					o.push(hex);
					hex = o.join('');
				}
				return hex;
			},
			/**
			 * Fix the values if the user enters a negative or high value in opacity input
			 * @method fixOpacity
			 * @private
			 */
			fixOpacity = function(op){
				return Math.min(100, Math.max(0, op));
			},
			/**
			 * Draw shapes for shape picker and advanced picker:
			 * Available shapes are:
			 * - square
			 * - round
			 * - roundsquare
			 * - triangle
			 * - diamond
			 * - pentagon
			 * - hexagon
			 * - heptagon
			 * - octagon
			 * - solid line
			 * - dashed line
			 * - dotted line 
			 * @method drawShapes
			 * @private
			 */
			drawShapes = function(cal, type){
				var canvas = $('<canvas class="shape ' + type + '" shape="' + type + '" width="50" height="50"></canvas>');

		    							var ctx = canvas.get(0).getContext("2d");
		    							ctx.fillStyle = "#000";
		    							ctx.strokeStyle='#000'; 
		    							switch(type){
			    							case "rectangle":
			    							case "square":
												ctx.fillRect(10,10,30,30);
												ctx.closePath();
											break;
											case "circle" :
			    							case "ellipse":
										        ctx.beginPath();
										        var x          = 25;               // x coordinate
										        var y          = 25;               // y coordinate
										        var radius     = 15;                    // Arc radius
										        var startAngle = 0;                     // Starting point on circle
										        var endAngle   = Math.PI+(Math.PI*2)/2; // End point on circle
										        var clockwise  = 2%2==0 ? false : true; // clockwise or anticlockwise
										    
										        ctx.arc(x,y,radius,startAngle,endAngle, clockwise);
										    	ctx.closePath();
										    	ctx.fill();
											break;
			    							case "triangle":
			    								 ctx.beginPath();
												// Draw a triangle location for each corner from x:y 100,110 -> 200,10 -> 300,110 (it will return to first point)
												ctx.moveTo(10,40);
												ctx.lineTo(25,10);
												ctx.lineTo(40,40);
												ctx.closePath();
												ctx.fill();
			    							break;
			    							case  "roundrectangle":
			    								var rectX = 10;
												var rectY = 10;
												var rectWidth = 30;
												var rectHeight = 30;
												var cornerRadius = 10;
	
			    								ctx.lineJoin = "round";
												ctx.lineWidth = cornerRadius;
												ctx.strokeRect(rectX+(cornerRadius/2), rectY+(cornerRadius/2), rectWidth-cornerRadius, rectHeight-cornerRadius);
												ctx.fillRect(rectX+(cornerRadius/2), rectY+(cornerRadius/2), rectWidth-cornerRadius, rectHeight-cornerRadius);
			    								ctx.closePath();
			    							break;
			    							case "solid":
			    								ctx.moveTo(10,25);
												ctx.lineTo(40,25);
												ctx.lineWidth = 3;
			    								ctx.closePath();
			    								ctx.stroke();
			    							break;
			    							case "dashed":
			    								
												var x0 = 10;
						    					var y0 = 25;
						    					var x1 = 40;
						    					var y1 = 25;
						    					var pattern = [4, 4];
						    					
						    					ctx.beginPath();
						    								   
												if (typeof ctx.setLineDash == "function"){
													ctx.setLineDash( params.strokeDash );
													ctx.webkitLineDash = ctx.mozDash = pattern;
												}else{
													
												    var length = Math.sqrt(Math.pow(x1 - x0, 2) + Math.pow(y1 - y0, 2));
												    var vector = {
												      x: (x1 - x0) / length,
												      y: (y1 - y0) / length
												    };
												    var dist = 0;
												    var i = 0;
												  
												   
												    while (dist < length) {
												      var dashLength = Math.min(pattern[i++ % pattern.length], length - dist);
												      var draw = (i % 2);
												      dist += dashLength;
												  
												      draw && ctx.moveTo(x0, y0);
												  
												      x0 += dashLength * vector.x;
												      y0 += dashLength * vector.y;
												  
												      draw && ctx.lineTo(x0, y0);
												    }
												}
												
												ctx.lineWidth = 3;
												ctx.closePath();
												ctx.stroke();
												
			    							break;
			    							case "dotted":

	    										var x0 = 10;
	    										var y0 = 25;
	    										var x1 = 40;
	    										var y1 = 25;
	    										var pattern = [1,6];
											    
											    ctx.beginPath();
											    
											    if (typeof ctx.setLineDash == "function"){
													ctx.setLineDash( params.strokeDash );
													ctx.webkitLineDash = ctx.mozDash = pattern;
												}else{
											    
												    var length = Math.sqrt(Math.pow(x1 - x0, 2) + Math.pow(y1 - y0, 2));
												    var vector = {
												      x: (x1 - x0) / length,
												      y: (y1 - y0) / length
												    };
												    var dist = 0;
												    var i = 0;
												  
												    while (dist < length) {
												      var dashLength = Math.min(pattern[i++ % pattern.length], length - dist);
												      var draw = (i % 2);
												      dist += dashLength;
												  
												      draw && ctx.moveTo(x0, y0);
												  
												      x0 += dashLength * vector.x;
												      y0 += dashLength * vector.y;
												  
												      draw && ctx.lineTo(x0, y0);
												    }
												}
											    
											   ctx.lineCap = 'round';
											   ctx.lineWidth = 4;
											   ctx.closePath();
											   ctx.stroke();
											   
			    							break;
			    							case "diamond":
			    								ctx.beginPath();
												ctx.moveTo(5,25);
												ctx.lineTo(22,10);
												ctx.lineTo(40,25);
												ctx.lineTo(22, 40);
												ctx.closePath();
												ctx.fill();
			    							break;
			    							case "tee":
			    								ctx.fillRect(20,10,10,30);
			    							break;
			    							case "none":
			    								ctx.beginPath();
										        var x          = 25;               // x coordinate
										        var y          = 25;               // y coordinate
										        var radius     = 12;                    // Arc radius
										        var startAngle = 0;                     // Starting point on circle
										        var endAngle   = Math.PI+(Math.PI*2)/2; // End point on circle
										        var clockwise  = 2%2==0 ? false : true; // clockwise or anticlockwise
										    
										        ctx.arc(x,y,radius,startAngle,endAngle, clockwise);
										    	ctx.lineWidth = 5;
										    
										    	ctx.moveTo(15,15);
												ctx.lineTo(35,35);
												ctx.closePath();
												ctx.stroke();
			    							break;
			    							case "pentagon":
			    								ctx.beginPath();
												ctx.moveTo(10,25);
												ctx.lineTo(25,10);
												ctx.lineTo(40,25);
												ctx.lineTo(33, 40);
												ctx.lineTo(17, 40);
												
												ctx.closePath();
												ctx.fill();
			    							break;
			    							case "hexagon":
			    								ctx.beginPath();
												ctx.moveTo(6,25);
												ctx.lineTo(13,10);
												ctx.lineTo(33,10);
												ctx.lineTo(40,25);
												ctx.lineTo(33, 40);
												ctx.lineTo(13, 40);
	
												ctx.closePath();
												ctx.fill();
			    							break;
			    							case "heptagon":
			    								ctx.beginPath();
												ctx.moveTo(8,31);
												ctx.lineTo(8,18);
												ctx.lineTo(23,10);
												ctx.lineTo(38,18);
												ctx.lineTo(38,31);
												ctx.lineTo(28, 40);
												ctx.lineTo(18, 40);
	
												ctx.closePath();
												ctx.fill();
			    							break;
			    							case "octagon":
			    								ctx.beginPath();
												ctx.moveTo(8,31);
												ctx.lineTo(8,18);
												ctx.lineTo(18,10);
												ctx.lineTo(28,10);
												ctx.lineTo(38,18);
												ctx.lineTo(38,31);
												ctx.lineTo(28, 40);
												ctx.lineTo(18, 40);
	
												ctx.closePath();
												ctx.fill();
			    							break;
			    							
	
										}
										cal.find(".shapepicker_shape").append(canvas.get(0));
			},

			/**
			 * Initialize input and return a picker object for each instance created
			 * @method restoreOriginal
			 * @return {object}
			 */
			restoreOriginal = function () {
				var cal = $(this).parent(),
					dataCal = cal.data('picker'),
					calGet0 = cal.get(0),
					col = dataCal.origColor,
					op = dataCal.origOpacity,
					sz = dataCal.origSize,
					nm = dataCal.origName,
					txt = dataCal.origText;
					sh = dataCal.origShape;
				
				dataCal.color = col;
				dataCal.opacity = op;
				dataCal.size = sz;
				dataCal.name = nm;
				dataCal.text = txt;
				dataCal.shape = sh;
				
				if($this.hasClass("color-chooser continuous_map") ){
					fillRGBFields(col, calGet0);
					fillHSBFields(col, calGet0);
					fillHexFields(col, calGet0);
					fillOpacityFields(op, calGet0);
					fillSizeFields(sz, calGet0);
					
					setSelectorColor(col, calGet0);
					setHue(col, calGet0);
					setNewColor(col, calGet0, op);
				}else if($this.hasClass("shape-chooser continuous_map")){
					fillSizeFieldsOnly(sz, calGet0);
				}else if($this.hasClass("annotation-chooser")){
					fillRGBFields(col, calGet0);
					fillHSBFields(col, calGet0);
					fillHexFields(col, calGet0);
					fillOpacityFields(op, calGet0);
					fillNameFields(nm, calGet0);
					fillTextFields(txt, calGet0);
					
					setSelectorColor(col, calGet0);
					setHue(col, calGet0);
					setNewColor(col, calGet0, op);
				}else if($this.hasClass("color-chooser")){
					fillRGBFields(col, calGet0);
					fillHSBFields(col, calGet0);
					fillHexFields(col, calGet0);
					fillOpacityFields(op, calGet0);
					
					setSelectorColor(col, calGet0);
					setHue(col, calGet0);
					setNewColor(col, calGet0, op);
				}

			};
		return {
			/**
			 * Test what kind of pickers/setters is required and initialize it with defaults values and events 
			 * @method init
			 * @method picker
			 * @param {object} opt the picker object with attributes values and events defined
			 */
			init: function (opt) {
				opt = $.extend({}, defaults, opt||{});
				var $this = $(this);
				//Set color
				if (typeof opt.color == 'string') {
					opt.color = hexToHsb(opt.color);
				} else if (opt.color.r != undefined && opt.color.g != undefined && opt.color.b != undefined) {
					opt.color = rgbToHsb(opt.color);
				} else if (opt.color.h != undefined && opt.color.s != undefined && opt.color.b != undefined) {
					opt.color = fixHSB(opt.color);
				} else {
					return this;
				}
				
				//For each selected DOM element
				return this.each(function () {
					//If the element does not have an ID
					if (!$(this).data('pickerId')) {							
							
						var options = $.extend({}, opt),
							type,
							cal,
							types_lines = ["dotted", "dashed", "solid"],
                			types_nodes = ["square", "roundrectangle", "ellipse", "triangle", "pentagon", "hexagon", "heptagon", "octagon"],
							types_arrows = ["diamond", "triangle", "square", "circle", "tee", "none"];
						
						
						//Generate and assign a random ID
						var id = 'picker_' + parseInt(Math.random() * 1000);
						
						if($this.hasClass("color-chooser continuous_map") ){
							options.origColor = opt.color;
							options.origOpacity = opt.opacity;
							options.origSize = opt.size;
							options.origShape = opt.shape;
							
							//Set the tpl's ID and get the HTML
							cal = $(tpl).attr('id', id);
							$this.data('pickerId', id);
							
						     switch ($this.attr('class')){
						     		case "color-chooser continuous_map nodes_colors advanced" :
						     		case "color-chooser continuous_map nodes_shapes advanced" :
							     	case "color-chooser continuous_map nodes_fill advanced" :
							    	 	cal.append($(node_shapepicker).addClass("advanced"));
							    	 	cal.append($(size_shapepicker).addClass("advanced"));
							    	 	
										for (var i = 0; i < types_nodes.length; i++) {
										
											type = types_nodes[i];
											drawShapes(cal, type);
											if(type==options.origShape){
												cal.find(".shape[shape="+type+"]").css("border-color", "red");
											}

										};

							     	break;
									case "color-chooser continuous_map nodes_border advanced":
										//cal.append($(line_shapepicker));
										// cal.append($(node_picker));
									break;
									case "color-chooser continuous_map nodes_label" :
											cal.append($(size_shapepicker).addClass("advanced"));
									break;
									case "color-chooser continuous_map edges_target advanced"  :
								    	 	cal.append($(arrow_shapepicker).addClass("advanced"));
																				
											for(var i = 0; i < types_arrows.length; i++){
	                   		 					type = types_arrows[i];     			
												drawShapes(cal, type);

												if(type==options.origShape){
													cal.find(".shape[shape="+type+"]").css("border-color", "red");
												}
	                						};
								    break;
									case "color-chooser continuous_map edges_source advanced"  :
								    	 	cal.append($(arrow_shapepicker).addClass("advanced"));
											for(var i = 0; i < types_arrows.length; i++){
	                   		 						type = types_arrows[i];        			
	                    		 					drawShapes(cal, type);

												if(type==options.origShape){
													cal.find(".shape[shape=none]").css("border-color", "red");
												}
	                						};
								    break;
									case "color-chooser continuous_map edges_lines":
									case "color-chooser continuous_map edges_shapes advanced":
									case "color-chooser continuous_map edges_colors advanced":
										cal.append($(line_shapepicker).addClass("advanced"));
										cal.append($(size_shapepicker).addClass("advanced"));
										for(var i = 0; i < types_lines.length; i++){
	                    							type = types_lines[i];
	                    							drawShapes(cal, type);

													if(type==options.origShape){
														cal.find(".shape[shape="+type+"]").css("border-color", "red");
													}
	                    								
	                					};
									break;
									case "color-chooser continuous_map edges_label" :
											cal.append($(size_shapepicker).addClass("advanced"));
										
									break;
						     };
						     
						    
						    //Add class according to layout
							cal.addClass('picker_'+options.layout+(options.submit?'':' picker_'+options.layout+'_ns'));
							//Add class if the color scheme is not default
							if(options.colorScheme != 'light') {
								cal.addClass('picker_'+options.colorScheme);
							}
							//Setup submit button
							cal.find('div.picker_submit').html(options.submitText).click(clickSubmit);
							//Setup input fields
							options.fields = cal.find('input').change(change).keydown(keyDown).blur(blur).focus(focus);
							 cal.find(".shape"+type).click(changeShape);
							cal.find('div.picker_field_arrs').mousedown(downIncrement).end().find('div.picker_current_color').click(restoreOriginal);
							//Setup hue selector
							options.selector = cal.find('div.picker_color').on('mousedown touchstart',downSelector);
							options.selectorIndic = options.selector.find('div.picker_selector_outer');
							//Store parts of the plugin
							options.el = this;
							options.hue = cal.find('div.picker_hue_arrs');
							var huebar = options.hue.parent();
							//Paint the hue bar
							var UA = navigator.userAgent.toLowerCase();
							var isIE = navigator.appName === 'Microsoft Internet Explorer';
							var IEver = isIE ? parseFloat( UA.match( /msie ([0-9]{1,}[\.0-9]{0,})/ )[1] ) : 0;
							var ngIE = ( isIE && IEver < 10 );
							var stops = ['#ff0000','#ff0080','#ff00ff','#8000ff','#0000ff','#0080ff','#00ffff','#00ff80','#00ff00','#80ff00','#ffff00','#ff8000','#ff0000'];
							if(ngIE) {
								var i, div;
								for(i=0; i<=11; i++) {
									div = $('<div></div>').attr('style','height:8.333333%; filter:progid:DXImageTransform.Microsoft.gradient(GradientType=0,startColorstr='+stops[i]+', endColorstr='+stops[i+1]+'); -ms-filter: "progid:DXImageTransform.Microsoft.gradient(GradientType=0,startColorstr='+stops[i]+', endColorstr='+stops[i+1]+')";');
									huebar.append(div);
								}
							} else {
								var stopList = stops.join(',');
								huebar.attr('style','background:-webkit-linear-gradient(top,'+stopList+'); background: -o-linear-gradient(top,'+stopList+'); background: -ms-linear-gradient(top,'+stopList+'); background:-moz-linear-gradient(top,'+stopList+'); -webkit-linear-gradient(top,'+stopList+'); background:linear-gradient(to bottom,'+stopList+'); ');
							}
							cal.find('div.picker_hue').on('mousedown touchstart',downHue);
							options.newColor = cal.find('div.picker_new_color');
							options.currentColor = cal.find('div.picker_current_color');
							
						    options.fields = cal.find('input').change(change).keydown(keyDown).blur(blur).focus(focus);
						   
						  
						   $(".picker").find(".shape").click(changeShape);
					   }else if($this.hasClass("color-chooser annotation-chooser")){
					   				options.origColor = opt.color;
									options.origOpacity = opt.opacity;;
									options.origName = opt.name;
									options.origText = opt.text;
									
									
									cal = $(tpl).attr('id', id);
					   				$this.data('pickerId', id);

					   				cal.addClass("sequencepicker").children(".colorpicker_color").append($(node_sequencepicker));
					   				//Add class according to layout
									cal.addClass('picker_'+options.layout+(options.submit?'':' picker_'+options.layout+'_ns'));
									//Add class if the color scheme is not default
									if(options.colorScheme != 'light') {
										cal.addClass('picker_'+options.colorScheme);
									}
									//Setup submit button
									cal.find('div.picker_submit').html(options.submitText).click(clickSubmit);
									//Setup input fields
									options.fields = cal.find('input, textarea').change(change).keydown(keyDown).blur(blur).focus(focus);
									cal.find('div.picker_field_arrs').mousedown(downIncrement).end().find('div.picker_current_color').click(restoreOriginal);
									//Setup hue selector
									options.selector = cal.find('div.picker_color').on('mousedown touchstart',downSelector);
									options.selectorIndic = options.selector.find('div.picker_selector_outer');
									//Store parts of the plugin
									options.el = this;
									options.hue = cal.find('div.picker_hue_arrs');
									var huebar = options.hue.parent();
									//Paint the hue bar
									var UA = navigator.userAgent.toLowerCase();
									var isIE = navigator.appName === 'Microsoft Internet Explorer';
									var IEver = isIE ? parseFloat( UA.match( /msie ([0-9]{1,}[\.0-9]{0,})/ )[1] ) : 0;
									var ngIE = ( isIE && IEver < 10 );
									var stops = ['#ff0000','#ff0080','#ff00ff','#8000ff','#0000ff','#0080ff','#00ffff','#00ff80','#00ff00','#80ff00','#ffff00','#ff8000','#ff0000'];
									if(ngIE) {
										var i, div;
										for(i=0; i<=11; i++) {
											div = $('<div></div>').attr('style','height:8.333333%; filter:progid:DXImageTransform.Microsoft.gradient(GradientType=0,startColorstr='+stops[i]+', endColorstr='+stops[i+1]+'); -ms-filter: "progid:DXImageTransform.Microsoft.gradient(GradientType=0,startColorstr='+stops[i]+', endColorstr='+stops[i+1]+')";');
											huebar.append(div);
										}
									} else {
										var stopList = stops.join(',');
										huebar.attr('style','background:-webkit-linear-gradient(top,'+stopList+'); background: -o-linear-gradient(top,'+stopList+'); background: -ms-linear-gradient(top,'+stopList+'); background:-moz-linear-gradient(top,'+stopList+'); -webkit-linear-gradient(top,'+stopList+'); background:linear-gradient(to bottom,'+stopList+'); ');
									}
									cal.find('div.picker_hue').on('mousedown touchstart',downHue);
									options.newColor = cal.find('div.picker_new_color');
									options.currentColor = cal.find('div.picker_current_color');
   
					   				
					   	}else if($this.hasClass("shape-chooser continuous_map")){
									
								options.origSize = opt.size;
								options.origShape = opt.shape;
	
								cal = $(tps).attr('id', id);
								$this.data('pickerId', id);
	
								switch ($this.attr('class')){
									case "shape-chooser continuous_map nodes_shapes":
										cal.append(node_shapepicker);
										for (var i = 0; i < types_nodes.length; i++) {
			
											type = types_nodes[i];
											drawShapes(cal, type);
		
											if(type==options.origShape){
												cal.find(".shape[shape="+type+"]").css("border-color", "red");
											}
										};
									break;
									case "shape-chooser continuous_map edges_target":
										cal.append(arrow_shapepicker);
										for (var i = 0; i < types_arrows.length; i++) {
			
											type = types_arrows[i];
											drawShapes(cal, type);
		
											if(type==options.origShape){
												cal.find(".shape[shape="+type+"]").css("border-color", "red");
											}
										};
									case "shape-chooser continuous_map edges_source":
										cal.append(arrow_shapepicker);
										for (var i = 0; i < types_arrows.length; i++) {
			
											type = types_arrows[i];
											drawShapes(cal, type);
		
											if(type==options.origShape){
												cal.find(".shape[shape="+type+"]").css("border-color", "red");
											}
										};

									break;
									case"shape-chooser continuous_map edges_lines":
										cal.append(line_shapepicker);
										for (var i = 0; i < types_lines.length; i++) {
			
											type = types_lines[i];
											drawShapes(cal, type);
		
											if(type==options.origShape){
												cal.find(".shape[shape="+type+"]").css("border-color", "red");
											}
										};
									break;
									
								};
								
								cal.append($(size_shapepicker));
								setTimeout(function() {
									cal.find(".shapepicker_size").append('<div class="picker_submit"></div>');
									//Setup submit button
									cal.find('div.picker_submit').html(options.submitText).click(clickSubmit);
									cal.find(".shape ").click(changeShape);
								}, 10);
								
								cal.find('div.picker_field_arrs').mousedown(downIncrement).end();
								
								options.fields = cal.find('input').change(changeSize).keydown(keyDown).blur(blur).focus(focus);
								
								
    
						 }else if ($this.hasClass("shape-chooser")) {
	
								options.origShape = opt.shape;
								cal = $(tps).attr('id', id);
								$this.data('pickerId', id);
	
								
								switch ($this.attr('class')){
									case "shape-chooser nodes_shapes":
										cal.append(node_shapepicker);
										for (var i = 0; i < types_nodes.length; i++) {
			
											type = types_nodes[i];
											drawShapes(cal, type);
										
											if(type==options.origShape){
												
												cal.find(".shape[shape="+type+"]").css("border-color", "red");
											}
										};
									break;
									case "shape-chooser edges_target":
										cal.append(arrow_shapepicker);
										for (var i = 0; i < types_arrows.length; i++) {
			
											type = types_arrows[i];
											drawShapes(cal, type);
		
											if(type==options.origShape){
												cal.find(".shape[shape="+type+"]").css("border-color", "red");
											}
										};
									case "shape-chooser edges_source":
										cal.append(arrow_shapepicker);
										for (var i = 0; i < types_arrows.length; i++) {
			
											type = types_arrows[i];
											drawShapes(cal, type);
		
											if(type==options.origShape){
												cal.find(".shape[shape="+type+"]").css("border-color", "red");
											}
										};

									break;
									case"shape-chooser edges_lines":
										cal.append(line_shapepicker);
										for (var i = 0; i < types_lines.length; i++) {
			
											type = types_lines[i];
											drawShapes(cal, type);
		
											if(type==options.origShape){
												cal.find(".shape[shape="+type+"]").css("border-color", "red");
											}
										};
									break;
									
								};
								setTimeout(function() {
									cal.find(".shape").click(changeShape);
	
								}, 10);
	
								//options.fields = cal.find('input').change(changeSize).keydown(keyDown).blur(blur).focus(focus);
								
								cal.find('div.picker_field_arrs').mousedown(downIncrement).end();
	
						}else if($this.hasClass("color-chooser")){
					   		
								options.origColor = opt.color;
								options.origOpacity = opt.opacity;
								
								cal = $(tpl).attr('id', id);
								$this.data('pickerId', id);
	
								cal = $(tpl).attr('id', id);
								$this.data('pickerId', id);
	
								//Add class according to layout
								cal.addClass('picker_' + options.layout + (options.submit ? '' : ' picker_' + options.layout + '_ns'));
								//Add class if the color scheme is not default
								if (options.colorScheme != 'light') {
									cal.addClass('picker_' + options.colorScheme);
								}
								//Setup submit button
								cal.find('div.picker_submit').html(options.submitText).click(clickSubmit);
								//Setup input fields
								options.fields = cal.find('input').change(change).blur(blur).focus(focus).keydown(keyDown);
								cal.find('div.picker_field_arrs').mousedown(downIncrement).end().find('div.picker_current_color').click(restoreOriginal);
								//Setup hue selector
								options.selector = cal.find('div.picker_color').on('mousedown touchstart', downSelector);
								options.selectorIndic = options.selector.find('div.picker_selector_outer');
								//Store parts of the plugin
								options.el = this;
								options.hue = cal.find('div.picker_hue_arrs');
								var huebar = options.hue.parent();
								//Paint the hue bar
								var UA = navigator.userAgent.toLowerCase();
								var isIE = navigator.appName === 'Microsoft Internet Explorer';
								var IEver = isIE ? parseFloat(UA.match( /msie ([0-9]{1,}[\.0-9]{0,})/ )[1]) : 0;
								var ngIE = (isIE && IEver < 10 );
								var stops = ['#ff0000', '#ff0080', '#ff00ff', '#8000ff', '#0000ff', '#0080ff', '#00ffff', '#00ff80', '#00ff00', '#80ff00', '#ffff00', '#ff8000', '#ff0000'];
								if (ngIE) {
									var i, div;
									for ( i = 0; i <= 11; i++) {
										div = $('<div></div>').attr('style', 'height:8.333333%; filter:progid:DXImageTransform.Microsoft.gradient(GradientType=0,startColorstr=' + stops[i] + ', endColorstr=' + stops[i + 1] + '); -ms-filter: "progid:DXImageTransform.Microsoft.gradient(GradientType=0,startColorstr=' + stops[i] + ', endColorstr=' + stops[i + 1] + ')";');
										huebar.append(div);
									}
								} else {
									var stopList = stops.join(',');
									huebar.attr('style', 'background:-webkit-linear-gradient(top,' + stopList + '); background: -o-linear-gradient(top,' + stopList + '); background: -ms-linear-gradient(top,' + stopList + '); background:-moz-linear-gradient(top,' + stopList + '); -webkit-linear-gradient(top,' + stopList + '); background:linear-gradient(to bottom,' + stopList + '); ');
								}
								cal.find('div.picker_hue').on('mousedown touchstart', downHue);
								options.newColor = cal.find('div.picker_new_color');
								options.currentColor = cal.find('div.picker_current_color'); 
					   	}
						
						//Store options and fill with default color
						cal.data('picker', options);
						
						var calGet0 = cal.get(0);
						if($this.hasClass("color-chooser continuous_map") ){
							fillRGBFields(options.color, calGet0);
							fillHSBFields(options.color, calGet0);
							fillHexFields(options.color, calGet0);
							fillOpacityFields(options.opacity, calGet0);
							fillSizeFields(options.size, calGet0);
							
							setHue(options.color, calGet0);
							setSelectorColor(options.color, calGet0);
							setCurrentColor(options.color, calGet0, options.opacity);
							setNewColor(options.color, calGet0, options.opacity);
						}else if($this.hasClass("shape-chooser continuous_map")){
							fillSizeFieldsOnly(options.size, calGet0);
						}else if($this.hasClass("annotation-chooser")){
							fillRGBFields(options.color, calGet0);
							fillHSBFields(options.color, calGet0);
							fillHexFields(options.color, calGet0);
							fillOpacityFields(options.opacity, calGet0);
							fillNameFields(options.name, calGet0);
							fillTextFields(options.text, calGet0);
							
							setHue(options.color, calGet0);
							setSelectorColor(options.color, calGet0);
							setCurrentColor(options.color, calGet0, options.opacity);
							setNewColor(options.color, calGet0, options.opacity);
						}else if($this.hasClass("color-chooser")){
							
							fillRGBFields(options.color, calGet0);
							fillHSBFields(options.color, calGet0);
							fillHexFields(options.color, calGet0);
							fillOpacityFields(options.opacity, calGet0);
							
							setHue(options.color, calGet0);
							setSelectorColor(options.color, calGet0);
							setCurrentColor(options.color, calGet0, options.opacity);
							setNewColor(options.color, calGet0, options.opacity);
						}

						//Append to body if flat=false, else show in place
						if (options.flat) {
							cal.appendTo(this).show();
							cal.css({
								position: 'relative',
								display: 'block'
							});
						} else {
							cal.appendTo(document.body);
							$(this).on(options.showEvent, show);
							cal.css({
								position:'absolute'
							});
						}
					}
				});
			},
			/**
			 * Shows the picker (called for show function).
			 * @method showPicker
			 * @for picker
			 */
			showPicker: function() {
				return this.each( function () {
					if ($(this).data('pickerId')) {
						show.apply(this);
					}
				});
			},
			/**
			 * Hides the picker (called for hide function).
			 * @method hidePicker
			 * @for picker
			 */
			hidePicker: function() {
				return this.each( function () {
					if ($(this).data('pickerId')) {
						$('#' + $(this).data('pickerId')).hide();
					}
				});
			},

			/**
			 * Sets a color as new and current (default), the size value and the text for annotation input and area.
			 * @method setValues
			 * @param {String} col the new color.
			 * @param {int} op the new opacity.
			 * @param {int} sz the new size.
			 * @param {String} nm the name of commentaries (annotation-chooser).
			 * @param {String} txt the text of commentaries  (annotation-chooser).
			 * @param {String} sh the shape wanted (shape_chooser).
			 * @param {String} setCurrent the current color.
			 * @for picker
			 */
			setValues: function(col, op, sz, nm, txt, sh, setCurrent) {
				
				setCurrent = (typeof setCurrent === "undefined") ? 1 : setCurrent;
				if(col != undefined){
					if (typeof col == 'string') {
						col = hexToHsb(col);
					} else if (col.r != undefined && col.g != undefined && col.b != undefined) {
						col = rgbToHsb(col);
					} else if (col.h != undefined && col.s != undefined && col.b != undefined) {
						col = fixHSB(col);
					} else {
						return this;
					}
				}
				return this.each(function(){
					
						var $this = $(this);
							
					if ($this.data('pickerId')) {
						var cal = $('#' + $(this).data('pickerId')), 
							calGet0 = cal.get(0),
							calData = cal.data('picker');
								
						calData.color = col;
						calData.opacity = op;
						calData.size = sz;
						calData.name = nm;
						calData.text = txt;
						calData.shape = sh;
						calData.origColor = col;
						calData.origOpacity = op;
						calData.origSize = sz;
						calData.origName = nm;
						calData.origText = txt;
						calData.origShape = sh;	
						if($this.hasClass("color-chooser continuous_map") ){
							fillRGBFields(col, calGet0);
							fillHSBFields(col, calGet0);
							fillHexFields(col, calGet0);
							fillOpacityFields(op, calGet0);
							fillSizeFields(sz, calGet0);
							
							setHue(col, calGet0);
							setSelectorColor(col, calGet0);
								
							setNewColor(col, calGet0, op);
							//cal.data('picker').onChangeColor.apply(cal.parent(), [col, hsbToHex(col), hsbToRgb(col), cal.data('picker').el, 1]);
							if(setCurrent) {
								setCurrentColor(col, calGet0, op);
							}
								
						}else if($this.hasClass("shape-chooser continuous_map")){
							fillSizeFieldsOnly(sz, calGet0);
							cal.find(".shape[shape="+sh+"]").css("border-color", "red");
						}else if($this.hasClass("color-chooser annotation-chooser")){
							fillRGBFields(col, calGet0);
							fillHSBFields(col, calGet0);
							fillHexFields(col, calGet0);
							fillOpacityFields(op, calGet0);
							fillNameFields(nm, calGet0);
							fillTextFields(txt, calGet0);
							
							setHue(col, calGet0);
							setSelectorColor(col, calGet0);
								
							setNewColor(col, calGet0, op);
							//cal.data('picker').onChangeColor.apply(cal.parent(), [col, hsbToHex(col), hsbToRgb(col), cal.data('picker').el, 1]);
							if(setCurrent) {
								setCurrentColor(col, calGet0, op);
							}
						}else if($this.hasClass("shape-chooser")){
							
							cal.find(".shape[shape="+sh+"]").css("border-color", "red");
						}else if($this.hasClass("color-chooser")){
							fillRGBFields(col, calGet0);
							fillHSBFields(col, calGet0);
							fillHexFields(col, calGet0);
							fillOpacityFields(op, calGet0);
							
							setHue(col, calGet0);
							setSelectorColor(col, calGet0);
								
							setNewColor(col, calGet0, op);
							//cal.data('picker').onChangeColor.apply(cal.parent(), [col, hsbToHex(col), hsbToRgb(col), cal.data('picker').el, 1]);
							if(setCurrent) {
								setCurrentColor(col, calGet0, op);
							}
						}	

					}
				});
			}
		};
	}();
	/**
	* Colors conversion hex to rgb
	* @method hexToRgb
	* @for picker
	*/
	var hexToRgb = function (hex) {
		var hex = parseInt(((hex.indexOf('#') > -1) ? hex.substring(1) : hex), 16);
		return {r: hex >> 16, g: (hex & 0x00FF00) >> 8, b: (hex & 0x0000FF)};
	};
	/**
	* Colors conversion hex to hsb
	* @method hexToHsb
	* @for picker
	*/
	var hexToHsb = function (hex) {
		return rgbToHsb(hexToRgb(hex));
	};
	/**
	* Colors conversion rgb to hsb
	* @method rgbToHsb
	* @for picker
	*/
	var rgbToHsb = function (rgb) {
		var hsb = {h: 0, s: 0, b: 0};
		var min = Math.min(rgb.r, rgb.g, rgb.b);
		var max = Math.max(rgb.r, rgb.g, rgb.b);
		var delta = max - min;
		hsb.b = max;
		hsb.s = max != 0 ? 255 * delta / max : 0;
		if (hsb.s != 0) {
			if (rgb.r == max) hsb.h = (rgb.g - rgb.b) / delta;
			else if (rgb.g == max) hsb.h = 2 + (rgb.b - rgb.r) / delta;
			else hsb.h = 4 + (rgb.r - rgb.g) / delta;
		} else hsb.h = -1;
		hsb.h *= 60;
		if (hsb.h < 0) hsb.h += 360;
		hsb.s *= 100/255;
		hsb.b *= 100/255;
		return hsb;
	};
	/**
	* Colors conversion hsb to rbg
	* @method hsbToRgb
	* @for picker
	*/
	var hsbToRgb = function (hsb) {
		var rgb = {};
		var h = hsb.h;
		var s = hsb.s*255/100;
		var v = hsb.b*255/100;
		if(s == 0) {
			rgb.r = rgb.g = rgb.b = v;
		} else {
			var t1 = v;
			var t2 = (255-s)*v/255;
			var t3 = (t1-t2)*(h%60)/60;
			if(h==360) h = 0;
			if(h<60) {rgb.r=t1;	rgb.b=t2; rgb.g=t2+t3}
			else if(h<120) {rgb.g=t1; rgb.b=t2;	rgb.r=t1-t3}
			else if(h<180) {rgb.g=t1; rgb.r=t2;	rgb.b=t2+t3}
			else if(h<240) {rgb.b=t1; rgb.r=t2;	rgb.g=t1-t3}
			else if(h<300) {rgb.b=t1; rgb.g=t2;	rgb.r=t2+t3}
			else if(h<360) {rgb.r=t1; rgb.g=t2;	rgb.b=t1-t3}
			else {rgb.r=0; rgb.g=0;	rgb.b=0}
		}
		return {r:Math.round(rgb.r), g:Math.round(rgb.g), b:Math.round(rgb.b)};
	};
	/**
	* Colors conversion rgb to hex
	* @method rgbToHex
	* @for picker
	*/
	var rgbToHex = function (rgb) {
		var hex = [
			rgb.r.toString(16),
			rgb.g.toString(16),
			rgb.b.toString(16)
		];
		$.each(hex, function (nr, val) {
			if (val.length == 1) {
				hex[nr] = '0' + val;
			}
		});
		return hex.join('');
	};
	/**
	* Colors conversion hsb to hex
	* @method hsbToHex
	* @for picker
	*/
	var hsbToHex = function (hsb) {
		return rgbToHex(hsbToRgb(hsb));
	};
	$.fn.extend({
		picker: picker.init,
		pickerHide: picker.hidePicker,
		pickerShow: picker.showPicker,
		pickerSetValues: picker.setValues
	});
	$.extend({
		picker:{ 
			rgbToHex: rgbToHex,
			rgbToHsb: rgbToHsb,
			hsbToHex: hsbToHex,
			hsbToRgb: hsbToRgb,
			hexToHsb: hexToHsb,
			hexToRgb: hexToRgb
		}
	});
})(jQuery);